/***********************************************************************
 AlignmentSeq.C
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#include "AlignmentSeq.H"
#include <iostream>
using namespace BOOM;


AlignmentSeq::AlignmentSeq(const BOOM::String &name,int ID,Symbol gapSymbol,
			   int begin)
  : name(name),
    isTargetTrack(false),
    ID(ID),
    gapSymbol(gapSymbol),
    begin(begin)
{
  // ctor
}



void AlignmentSeq::setTarget()
{
  isTargetTrack=true;
}



bool AlignmentSeq::isTarget() const
{
  return isTargetTrack;
}



const BOOM::String &AlignmentSeq::getName() const
{
  return name;
}



int AlignmentSeq::getBegin() const
{
  return begin;
}



int AlignmentSeq::getLength() const
{
  return seq.getLength();
}



void AlignmentSeq::addHSP(int begin,const BOOM::Sequence &hsp)
{
  if(seq.getLength()==0) this->begin=begin;
  seq.append(hsp);
}



void AlignmentSeq::extendToLength(int length)
{
  int L=seq.getLength();
  if(L<length)
    {
      int delta=length-L;
      for(int i=0 ; i<delta ; ++i)
	seq.append(gapSymbol);
    }
}



const BOOM::Sequence &AlignmentSeq::getSeq() const
{
  return seq;
}



BOOM::Sequence &AlignmentSeq::getSeq() {
  return seq;
}



void AlignmentSeq::deleteSequence()
{
  seq.clear();
}



int AlignmentSeq::getID() const
{
  return ID;
}



void AlignmentSeq::append(const AlignmentSeq &other)
{
  seq.append(other.seq);
}






