/***********************************************************************
 AlignmentSeq.H
 bmajoros@duke.edu

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_AlignmentSeq_H
#define INCL_AlignmentSeq_H
#include "String.H"
#include "Sequence.H"


// a sequence (i.e., "track") in an alignment

namespace BOOM {
class AlignmentSeq 
{
public:
  AlignmentSeq(const BOOM::String &name,int ID,Symbol gapSymbol,int begin=0);
  const BOOM::String &getName() const;
  int getID() const;
  void setID(int id) {ID=id;}
  int getBegin() const; // relative to source genome, not to alignment!
  int getLength() const;
  void addHSP(int begin,const BOOM::Sequence &seq);
  void extendToLength(int length);
  const BOOM::Sequence &getSeq() const;
  BOOM::Sequence &getSeq();
  inline Symbol &operator[](int);
  void deleteSequence();
  void setTarget();
  bool isTarget() const;
  void append(const AlignmentSeq &);
private:
  BOOM::String name;
  int begin; // relative to source genome, not to alignment!
  BOOM::Sequence seq;
  bool isTargetTrack;
  int ID;
  Symbol gapSymbol;
};
    
 
    
Symbol &AlignmentSeq::operator[](int index)
{
    return seq[index];
}

}

#endif
