/***********************************************************************
 AlignmentTrack.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_AlignmentTrack_H
#define INCL_AlignmentTrack_H
#include "String.H"

namespace BOOM {
class AlignmentTrack
{
public:
  AlignmentTrack(const BOOM::String &name,int ID);
  const BOOM::String &getName() const;
  void rename(const BOOM::String &newName);
  int getID() const;
  void setID(int id) {ID=id;}
  int getBegin() const; // relative to source genome, not to alignment!
  int getLength() const;
  int getNonGapLength() const; // number of non-gap entries in track
  void addHSP(int begin,const BOOM::String &seqStr);
  void extendToLength(int length);
  const BOOM::String &getSeq() const;
  inline char &operator[](int);
  void deleteSequence();
  void setTarget();
  bool isTarget() const;
  void append(const AlignmentTrack &);
  void toupper();
private:
  BOOM::String name;
  int begin; // relative to source genome, not to alignment!
  BOOM::String seqStr;
  bool isTargetTrack;
  int ID;
};
    
    
    
char &AlignmentTrack::operator[](int index)
{
    return seqStr[index];
}

}

#endif
