/****************************************************************
 AlphabetMap.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "AlphabetMap.H"
#include "DnaAlphabet.H"
#include "PureDnaAlphabet.H"
using namespace std;
using namespace BOOM;


const DropNMapping DropNMapping::global;


AlphabetMap::AlphabetMap(Alphabet &from,Alphabet &to)
  : theMap(from.getNumElements()),
    inverseMap(to.getNumElements())
{
}



AlphabetMap::AlphabetMap(int domainSize,int rangeSize)
  : theMap(domainSize),
    inverseMap(rangeSize)
{
}



AlphabetMap::AlphabetMap(const AlphabetMap &other)
  : theMap(other.theMap),
    inverseMap(other.inverseMap)
{
}



Symbol AlphabetMap::map(Symbol s) const
{
  return theMap[s];
}



Symbol AlphabetMap::unmap(Symbol s) const
{
  return inverseMap[s];
}



int AlphabetMap::getDomainSize() const
{
  return theMap.size();
}



int AlphabetMap::getRangeSize() const
{
  return inverseMap.size();
}



void AlphabetMap::defineMapping(Symbol from,Symbol to)
{
  theMap[from]=to;
}



void AlphabetMap::defineInverse(Symbol from,Symbol to)
{
  inverseMap[from]=to;
}



AlphabetIdentityMap::AlphabetIdentityMap(Alphabet &a)
  : AlphabetMap(a,a)
{
  int n=a.getNumElements();
  for(Symbol i=0 ; i<n ; ++i)
    {
      defineMapping(i,i);
      defineInverse(i,i);
    }
}



DropNMapping::DropNMapping()
  : AlphabetMap(5,4)
{
  Symbol fA=0;//DnaAlphabet::global.lookup('A');
  Symbol fC=1;//DnaAlphabet::global.lookup('C');
  Symbol fG=2;//DnaAlphabet::global.lookup('G');
  Symbol fN=3;//DnaAlphabet::global.lookup('N');
  Symbol fT=4;//DnaAlphabet::global.lookup('T');

  Symbol tA=0;//DnaAlphabet::global.lookup('A');
  Symbol tC=1;//DnaAlphabet::global.lookup('C');
  Symbol tG=2;//DnaAlphabet::global.lookup('G');
  Symbol tT=3;//DnaAlphabet::global.lookup('T');

  defineMapping(fA,tA);
  defineMapping(fC,tC);
  defineMapping(fG,tG);
  defineMapping(fN,INVALID_SYMBOL);
  defineMapping(fT,tT);

  defineInverse(tA,fA);
  defineInverse(tC,fC);
  defineInverse(tG,fG);
  defineInverse(tT,fT);
}

