/***********************************************************************
 BitSet.H
 bmajoros@duke.edu

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_BitSet_H
#define INCL_BOOM_BitSet_H
using namespace std;
#include <stdio.h>

namespace BOOM {
class BitSet 
{
public:
    BitSet(unsigned long Size=0);  
    virtual ~BitSet();
    void operator =(BitSet &RValue);   
    void operator -=(BitSet &otherSet);  
    void operator +=(BitSet &otherSet);  
    void operator *=(BitSet &otherSet);  
    bool operator ==(BitSet &otherSet);
    bool operator !=(BitSet &otherSet);
    BitSet *operator -(BitSet &otherSet);  
    BitSet *operator +(BitSet &otherSet);  
    BitSet *operator *(BitSet &otherSet);  
    void intersect(BitSet &otherSet,BitSet &into);
    void unionWith(BitSet &otherSet,BitSet &into);
    bool isMember(unsigned long BitNumber);
    unsigned long getIthMember(unsigned long i);
    unsigned long cardinality();
    bool isEmpty();
    unsigned long getMaxSize();
    void addAll();  
    void addMember(unsigned long BitNumber);  
    void complement();
    void getRawBytes(unsigned char *&,unsigned long &maxSize,
                     unsigned long &numBytes);
    void load(FILE *fp);
    void purge();                
    void removeMember(unsigned long BitNumber);  
    void replaceRawBytes(unsigned char *,unsigned long maxSize,
                         unsigned long numBytes);
    void save(FILE *fp);  
    void setSize(unsigned long Size);
  void swapBits(unsigned long bitNumber1,unsigned long bitNumber2);
protected:
  unsigned long NumBytes;  
  unsigned long MaxSize;   
  unsigned char *TheBitMap;
};
}

#endif
