/***********************************************************************
 ElasVector.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_ElasVector_H
#define INCL_BOOM_ElasVector_H
#include "Vector.H"
using namespace std;

namespace BOOM {
template<class T>
class ElasVector
{
public:
  ElasVector(const T &nullInitializer);
  T &operator[](int);
  int size();
private:
  const T &nullInitializer;
  BOOM::Vector<T> theVector;
};

class ElasDblVect : public BOOM::ElasVector<double>
{
public:
  ElasDblVect();
};

class ElasFloatVect : public BOOM::ElasVector<float>
{
public:
  ElasFloatVect();
};

class ElasIntVect : public BOOM::ElasVector<int>
{
public:
  ElasIntVect();
};

class ElasCharVect : public BOOM::ElasVector<char>
{
public:
  ElasCharVect();
};

class ElasLongVect : public BOOM::ElasVector<long>
{
public:
  ElasLongVect();
};
}


BOOM::ElasDblVect() : BOOM::ElasVector<double>(0) 
{
}



BOOM::ElasFloatVect() : BOOM::ElasVector<float>(0) 
{
}



BOOM::ElasIntVect() : BOOM::ElasVector<int>(0) 
{
}



BOOM::ElasCharVect() : BOOM::ElasVector<char>('\0') 
{
}



BOOM::ElasLongVect() : BOOM::ElasVector<long>(0) 
{
}



template<class T>
T &BOOM::ElasVector<T>::operator[](int index)
{
  int sz=theVector.size();
  if(index>=sz)
    for(int i=sz ; i<=index ; ++i)
      theVector.push_back(nullInitializer);
  return theVector[index];
}



template<class T>
BOOM::ElasVector<T>::ElasVector(const T &nullInitializer)
  : nullInitializer(nullInitializer)
{
}



template<class T>
int BOOM::ElasVector<T>::size()
{
  return theVector.size();
}

#endif
