/***********************************************************************
 FastaReader.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_FastaReader_H
#define INCL_BOOM_FastaReader_H
#include "Vector.H"
#include "String.H"
#include "File.H"
#include "DnaAlphabet.H"
#include "AminoAlphabet.H"
#include "Regex.H"
using namespace std;

/*
  Reads a FASTA file into memory, converts sequence to uppercase,
  and masks strange characters (anything not in the alphabet) using N's.
 */
namespace BOOM {
class FastaReader
{
public:
  FastaReader(const BOOM::String &filename,Alphabet & =DnaAlphabet::global);
  BOOM::Vector< pair<BOOM::String,BOOM::String> > *readAll();
  bool nextSequence(BOOM::String &defline,BOOM::String &sequence);
  static void parseDefline(const BOOM::String &defline,BOOM::String &id,
			   BOOM::String &remainder);
  static BOOM::String getId(const BOOM::String &defline);
  void close() {file.close();}
private:
  BOOM::String cache;
  BOOM::File file;
  Alphabet &alphabet;
  static BOOM::Regex deflineRegex;

  void maskStrangeChars(BOOM::String &);
};
}

#endif
