/****************************************************************
 GaussianDistribution.C
 william.majoros@duke.edu

 This is open-source software, governed by the ARTISTIC LICENSE 
 (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "GaussianDistribution.H"
#include <gsl/gsl_rng.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_randist.h>
#include "Random.H"
using namespace std;
using namespace GSL;


GaussianDistribution::GaussianDistribution(double mean,
                                           double standardDeviation)
    : mean(mean),
      SD(standardDeviation)
{
    // ctor
}



double GaussianDistribution::random() {
    return mean+gsl_ran_gaussian(Random::getGenerator(),SD);
}



double GaussianDistribution::probabilityEquals(double value) {
    return gsl_ran_gaussian_pdf(value-mean,SD);
}



double GaussianDistribution::probabilityLessThan(double value) {
    return gsl_cdf_gaussian_P(value-mean,SD);
}



double GaussianDistribution::probabilityGreaterThan(double value) {
    return gsl_cdf_gaussian_Q(value-mean,SD);
}



double GaussianDistribution::criticalValueLessThan(double P) {
    return mean+gsl_cdf_gaussian_Pinv(P,SD);
}



double GaussianDistribution::criticalValueGreaterThan(double P) {
    return mean+gsl_cdf_gaussian_Qinv(P,SD);
}





