/****************************************************************
 Matrix.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Matrix_H
#define INCL_Matrix_H
#include <iostream>
#include <gsl/gsl_eigen.h>
#include "../TNT/tnt_array2d.h"
#include <complex>
#include "../Vector.H"
#include "../String.H"
using namespace std;

namespace GSL {

class Vector;

class Matrix
{
public:
    Matrix(int rows=1,int cols=1); // tested
    Matrix(const GSL::Matrix &); // tested
    Matrix(const TNT::Array2D<double> &);
    Matrix(const BOOM::String &filename);
    Matrix(istream &);
    virtual ~Matrix();
    void load(istream &);
    void load(const BOOM::String &filename);
    void becomeIdentity(); // tested
    double operator()(int row,int col) const;
    double &operator()(int row,int col); // tested
    const GSL::Matrix &operator=(const GSL::Matrix &);//tested
    bool invert(GSL::Matrix &resultingMatrix) const;//tested
    inline int getNumColumns() const;//tested
    inline int getNumRows() const;//tested
    void getColumn(int column,GSL::Vector &into) const;
    void getRow(int row,GSL::Vector &into) const;
    void setColumn(int column,const GSL::Vector &);
    void setRow(int row,const GSL::Vector &);
    void printOn(ostream &) const;//tested
    void setAllTo(double d);//tested
    void times(const GSL::Matrix &,GSL::Matrix &resultMatrix) const;//tested
    void power(int power,GSL::Matrix &result);
    void transpose(); // tested
    void transpose(GSL::Matrix &) const;
    void resize(int,int);//tested
    void multiply(const GSL::Vector &by,GSL::Vector &result) const;
    double determinant() const;//tested
    bool isZero(double epsilon=1e-15) const;//tested
    bool isIdentity(double epsilon=1e-15) const;//tested
    void getEigenVectors(GSL::Matrix &vectors, // <--columns are eigenvectors
                         GSL::Vector &values,
                         GSL::Vector &imagParts) const;
    void getEigenVectors(GSL::Matrix &vectors, // <--columns are eigenvectors
                         BOOM::Vector<complex<double> >&
                         eigenvalues);
    static void sortEigen(GSL::Matrix &vectors,GSL::Vector &values);
    static void sortEigenAbs(GSL::Matrix &vectors,GSL::Vector &values);
    bool sameSize(const GSL::Matrix &) const;
    TNT::Array2D<double> *convertToTNT() const;
    GSL::Matrix &operator=(const TNT::Array2D<double> &);
    void applyThreshold(double);
    void add(const GSL::Matrix &other,GSL::Matrix &result) const;
    void subtract(const GSL::Matrix &other,GSL::Matrix &result) const;
    void scale(double factor);
private:
    gsl_matrix *M;
};

ostream &operator<<(ostream &,const GSL::Matrix &);
istream &operator>>(istream &,GSL::Matrix &);


int Matrix::getNumColumns() const
{
  return M->size2;
}



int Matrix::getNumRows() const
{
  return M->size1;
}




}

#endif
