/****************************************************************
 Vector.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Vector_H
#define INCL_Vector_H
#include <iostream>
#include <gsl/gsl_vector.h>
#include "../TNT/tnt_array1d.h"
using namespace std;


namespace GSL {

class Vector
{
public:
  Vector(int dim=1);
  Vector(const GSL::Vector &);
  Vector(TNT::Array1D<double> &);
  Vector(gsl_vector *,bool shouldDelete=false);
  Vector(const gsl_vector *);
  virtual ~Vector();
  int getDim() const;
  double &operator[](int);
  double operator[](int) const;
  void setAllTo(double);
  void add(const GSL::Vector &other,GSL::Vector &result) const;
  void accumulate(const GSL::Vector &other);
  void sub(const GSL::Vector &other,GSL::Vector &result) const;
  void scale(double factor);
  double dotProduct(const GSL::Vector &) const;
  GSL::Vector &operator=(const GSL::Vector &);
  GSL::Vector &operator=(const TNT::Array1D<double> &);
  bool operator<(const GSL::Vector &) const;
  bool operator==(const GSL::Vector &) const;
  double norm() const;
  bool isNull() const;
  void addConstant(double,GSL::Vector &result) const;
  gsl_vector *peek();
  const gsl_vector *peek() const;
  void printOn(ostream &) const;
  void loadFrom(istream &);
  void resize(int);
  void applyThreshold(double);
private:
  gsl_vector *V;
  bool shouldDelete;
};

ostream &operator<<(ostream &,const GSL::Vector &);
istream &operator>>(istream &,GSL::Vector &);
}

#endif

