/***********************************************************************
 GffFeature.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_GffFeature_H
#define INCL_BOOM_GffFeature_H
#include "Vector.H"
#include "String.H"
#include "Regex.H"
#include "Comparator.H"
using namespace std;

namespace BOOM {

  enum GffFeatureType {
    GFT_INITIAL_EXON,
    GFT_INTERNAL_EXON,
    GFT_FINAL_EXON,
    GFT_SINGLE_EXON,
    GFT_INTRON,
    GFT_CDS,
    GFT_EXON,
    GFT_UNKNOWN
  };

class GffFeature
{
public:
  GffFeature(const String &rawLine,
	     const String &substrate,
	     const String &source,
	     const String &featureType,
	     int begin,int end,double score,bool hasScore,char strand,
	     int frame,bool hasFrame);
  GffFeature(const String &rawLine);
  String toGff() const;
  Vector<String> &getExtraFields();
  bool hasExtraFields() const;
  bool isFramed() const;  
  bool isScored() const;  
  bool isStranded() const;  
  char getStrand() const;  
  const String &getFeatureType() const;
  GffFeatureType interpretFeatureType() const {return featureTypeEnum;}
  const String &getRawField(int index);
  const String &getRawLine() const;
  const String &getSource() const;
  const String &getSubstrate() const;
  double getScore() const;
  int getBegin() const;  // 0-based, always less than end
  int getEnd() const;    // 0-based, always greater than begin
  int getFrame() const;  
  BOOM::String getTranscriptId() const;
  void updateFeatureTypeEnum();
private:
  int frame;  
  int begin, end;  
  double score;
  char strand;  
  bool hasScore;  
  bool hasFrame;  
  Vector<String> extraFields, allFields;
  String substrate;  
  String source;  
  String rawLine;  
  String featureType;  
  GffFeatureType featureTypeEnum;
  static Regex transgrpRegex, transcriptIdRegex;
  void parseLine(const String &);
};


class GffFeatureComparator : public Comparator<GffFeature*>
{
public:
  bool less(GffFeature *&,GffFeature *&);
  bool greater(GffFeature *&,GffFeature *&); 
  bool equal(GffFeature *&,GffFeature *&);
};

}
using namespace BOOM;

ostream &operator<<(ostream&,GffFeature&);

#endif
