/***********************************************************************
 Map.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_Map_H
#define INCL_BOOM_Map_H
#include <map>
#include <iostream>
using namespace std;

namespace BOOM {
template<class T>
struct DefaultMapComparator
{
  bool operator()(const T &a,const T &b){return a<b;}
};



template<class KeyType,class ValueType,
	 class Comp=DefaultMapComparator<KeyType> >
class Map : public map<KeyType,ValueType,Comp>
{
public:
  bool isDefined(const KeyType &key);
  void undefine(const KeyType &key) {erase(key);}
  inline int numElements();
  inline bool isEmpty() {return numElements()==0;}
  void printOn(ostream &);
  void readFrom(istream &);
};



template<class KeyType,class ValueType,class Comp>
ostream &operator<<(ostream &os,Map<KeyType,ValueType,Comp> &m)
{
  m.printOn(os);
  return os;
}



template<class KeyType,class ValueType,class Comp>
istream &operator>>(istream &is,Map<KeyType,ValueType,Comp> &m)
{
  m.readFrom(is);
  return is;
}
}



template<class KeyType,class ValueType,class Comp>
bool BOOM::Map<KeyType,ValueType,Comp>::isDefined(const KeyType &key)
{
  return this->find(key)!=end();
}



template<class KeyType,class ValueType,class Comp>
inline int BOOM::Map<KeyType,ValueType,Comp>::numElements() 
{
  return size();
}



template<class KeyType,class ValueType,class Comp>
void BOOM::Map<KeyType,ValueType,Comp>::printOn(ostream &os)
{
  int n=size();
  os<<n<<endl;
  typename Map<KeyType,ValueType,Comp>::iterator cur=this->begin(), 
    end=this->end();
  for(; cur!=end ; ++cur)
    {
      pair<const KeyType,ValueType> &p=*cur;
      os<<p.first<<'\t'<<p.second<<endl;
    }
}



template<class KeyType,class ValueType,class Comp>
void BOOM::Map<KeyType,ValueType,Comp>::readFrom(istream &is)
{
  clear();
  int n;
  is>>n;
  KeyType key;
  ValueType value;
  for(int i=0 ; i<n ; ++i)
    {
      is>>key>>value;
      (*this)[key]=value;
    }
}


#endif
