/***********************************************************************
 MultSeqAlignment.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_MultSeqAlignment_H
#define INCL_MultSeqAlignment_H
#include "String.H"
#include "Vector.H"
#include "Map.H"
#include "AlignmentSeq.H"
#include "Regex.H"
#include "MultiAlignment.H"
#include "Alphabet.H"

namespace BOOM {
class MultSeqAlignment
{
public:
  MultSeqAlignment(const MultiAlignment &,Alphabet &,Symbol gapSymbol);
  virtual ~MultSeqAlignment();
  AlignmentSeq &findOrCreateTrack(const BOOM::String &trackName);
  void addTrack(AlignmentSeq *);
  AlignmentSeq &getIthTrack(int i);
  void releaseTrack(int i); // won't delete it; set it to NULL
  AlignmentSeq &getTrackByName(const BOOM::String &);
  int getNumTracks() const;
  int getLength() const;
  void deleteSequences();
  void save(const BOOM::String &filename);
  void save(ostream &);
  void printSlice(ostream &os,int begin,int length,char strand,
		  int lineLength) const;
  MultSeqAlignment *getSlice(int begin,int length);
  bool containsGap();
  static int rightmostGapPos(const Sequence &m,Symbol gapSymbol);
  Symbol getGapSymbol() const;
  void deleteTargetGaps(int trackID);
  int getPhase() const {return phase;}
protected:
  MultSeqAlignment(Alphabet &,Symbol gapSymbol);
  BOOM::Vector<AlignmentSeq*> tracks;
  BOOM::Map<BOOM::String,int> nameToTrack;
  int length, phase;
  Alphabet &alphabet;
  Symbol gapSymbol;
  
  void convert(const MultiAlignment &,Symbol gapSymbol);
};
}

#endif
