/****************************************************************
 PrefixSumArray.H
 william.majoros@duke.edu

 Copyright 2007 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_PrefixSumArray_H
#define INCL_PrefixSumArray_H
#include <iostream>
#include "Array1D.H"
using namespace std;
using namespace BOOM;


// ================================================================
//                        template PrefixSumArray
// ================================================================
template<class T>
class PrefixSumArray
{
public:
  PrefixSumArray(int length=0,int offset=0);
  void load(const String &filename);
  void setOffset(int offset); // added to all coordinates
  int getLength() const;
  int getOffset() const;
  void resize(int length);
  void addScore(int zeroBasedPosition,T scoreForThisPosition);
  T getIntervalScore(int begin,int end_inclusive); // end is inclusive!!!
  void scaleBy(double factor);

protected:
  Array1D<double> A;
  int offset; // which position in the sequence is represented by A[0]
};




// ================================================================
//                        template methods
// ================================================================

template<class T>
PrefixSumArray<T>::PrefixSumArray(int length,int offset)
  : A(length), offset(offset)
{
  // ctor
}



template<class T>
void PrefixSumArray<T>::addScore(int pos,T score)
{
  pos-=offset;
  if(pos==0) A[0]=score;
  else A[pos]=A[pos-1]+score;
}



template<class T>
T PrefixSumArray<T>::getIntervalScore(int begin,int end)
{
  // end is inclusive!!!

  begin-=offset;
  end-=offset;

  if(end<begin) 
    throw String("PrefixSumArray::getIntervalScore() : negative interval");
  return A[end]-A[begin-1];
}



template<class T>
int PrefixSumArray<T>::getLength() const
{
  return A.size();
}



template<class T>
void PrefixSumArray<T>::resize(int length)
{
  A.resize(length);
}



template<class T>
int PrefixSumArray<T>::getOffset() const
{
  return offset;
}



template<class T>
void PrefixSumArray<T>::setOffset(int offset)
{
  this->offset=offset;
}



template<class T>
void PrefixSumArray<T>::load(const String &filename)
{
  File f(filename);
  int numBytes=f.getSize();
  int numDoubles=numBytes/sizeof(double);
  resize(numDoubles);
  for(int i=0 ; i<numDoubles ; ++i)
    A[i]=f.readDouble();
  f.close();
}



template<class T>
void PrefixSumArray<T>::scaleBy(double factor)
{
  int n=A.size();
  for(int i=0 ; i<n ; ++i)
    A[i]*=factor;
}


#endif

