/***********************************************************************
 Sequence.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_Sequence_H
#define INCL_Sequence_H
using namespace std;
#include <iostream>
#include <ctype.h>
#include "String.H"
#include "Vector.H"
#include "Alphabet.H"
#include "AlphabetMap.H"
#include "Symbol.H"
#include "Regex.H"

namespace BOOM {
class Sequence
{
public:
  static Sequence *load(const BOOM::String &filename,Alphabet &,
			BOOM::String &substrateId);
  static Sequence *load(const BOOM::String &filename,Alphabet &);
  Sequence(const BOOM::String &,Alphabet &);
  Sequence();
  Sequence(const char *,Alphabet &);
  Sequence(Symbol);
  Sequence(Symbol,int repetitions);
  virtual ~Sequence() {}
  void copyFrom(const BOOM::String &,Alphabet &);
  void setPhase(int);
  int getPhase() const;
  void append(Symbol);
  void append(const Sequence &);
  void prepend(Symbol);
  bool operator<(const Sequence &) const;
  Sequence operator+(const Sequence &);
  Sequence operator+(Symbol);
  void operator+=(const Sequence &);
  void operator+=(Symbol);
  String operator()(const Alphabet &) const;
  Symbol operator [](int) const;
  Symbol &operator [](int);
  const Sequence &operator=(const Sequence &);
  bool operator==(const Sequence &) const;
  int getLength() const;
  void printOn(ostream &) const;
  void printOn(ostream &,const Alphabet &) const;
  BOOM::String *toString(const Alphabet &,int startingAt=0) const;
  void toString(const Alphabet &,int startingAt,int length,BOOM::String &) const;
  void clear();
  virtual void getSubsequence(int begin,int len,const Sequence &) const;
  bool subsequenceOccursAt(const Sequence &,int at) const;
  Sequence *reverseComplement(Alphabet &) const;
  void reverseComplement(Alphabet &,Sequence &) const;
  bool isEmpty() const {return symbols.isEmpty();}
  int countOccurrences(Symbol);
  bool containsSymbol(Symbol);
  void insert(Symbol,int pos);
  void insert(const Sequence &,int pos);
  void cut(int begin,int len);
  unsigned asInt(const AlphabetMap &,int begin=0,int len=-1) const; // encode
  void fromInt(unsigned code,int seqLength,const AlphabetMap &); // decode
private:
  static BOOM::Regex deflineRegex;
  BOOM::Vector<Symbol> symbols;
  int phase;

  void append(Alphabet &,const char *);
};
}

Sequence operator+(Symbol,Symbol);
ostream &operator<<(ostream &,const Sequence &);

#endif
