/***********************************************************************
 Set.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_Set_H
#define INCL_BOOM_Set_H
#include <set>
#include <algorithm>
using namespace std;

namespace BOOM {
template< class T, class C = less<T> >
class Set : public set<T,C>
{
public:
  Set() {}
  bool operator+=(const BOOM::Set<T,C> &);
  bool operator+=(const T &);
  bool isMember(const T &);
  void remove(const T &t);
  BOOM::Set<T,C> *intersectWith(BOOM::Set<T,C> &);
  void intersectWith(BOOM::Set<T,C> &,BOOM::Set<T,C> &destination);
  inline bool isEmpty() const {return empty();}
};
}
using namespace BOOM;


template<class T, class C>
void BOOM::Set<T,C>::intersectWith(BOOM::Set<T,C> &other,
				 BOOM::Set<T,C> &destination)
{
  set_intersection(begin(),end(),other.begin(),other.end(),
		   inserter(destination,destination.begin()));
}



template<class T, class C>
BOOM::Set<T,C> *BOOM::Set<T,C>::intersectWith(BOOM::Set<T,C> &other)
{
  BOOM::Set<T,C> *newSet=new BOOM::Set<T,C>;
  intersectWith(other,*newSet);
  return newSet;
}



template<class T, class C>
bool BOOM::Set<T,C>::isMember(const T &t)
{
  return this->find(t)!=end();
}



template<class T, class C>
bool BOOM::Set<T,C>::operator+=(const T &other)
{
  if(!isMember(other))
    {
      insert(other);
      return true;
    }
  return false;
}



template<class T, class C>
bool BOOM::Set<T,C>::operator+=(const BOOM::Set<T,C> &other)
{
  bool changes=false;
  typename BOOM::Set<T,C>::iterator cur=other.begin(), end=other.end();
  for(; cur!=end ; ++cur)
    if(!isMember(*cur))
      {
	insert(*cur);
	changes=true;
      }
  return false;
}



template<class T, class C>
void BOOM::Set<T,C>::remove(const T &t) 
{
  erase(t);
}

#endif
