/***********************************************************************
 SparseGraph.C
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#include <iostream>
#include "SparseGraph.H"
using namespace std;



BOOM::SparseGraph::SparseGraph(int numVertices)
  : numEdges(0)
{
  for(int i=0 ; i<numVertices ; ++i) 
    addVertex();
}



Neighborhood *BOOM::SparseGraph::getNeighborsOf(VertexId vertexId,
						bool &shouldDelete) const
{
  shouldDelete=false;
  return &const_cast<SparseGraph*>(this)->neighborhoods[vertexId];
}



VertexId BOOM::SparseGraph::addVertex()
{
  VertexId id=neighborhoods.size();
  neighborhoods.push_back(Neighborhood());
  return id;
}



bool BOOM::SparseGraph::areAdjacent(VertexId v1,VertexId v2) const
{
  const Neighborhood &v1neighbors=neighborhoods[v1];
  bool notFound=v1neighbors.find(v2)==v1neighbors.end();
  return !notFound;
}



int BOOM::SparseGraph::getDegree(VertexId vertex) const
{
  int degree=neighborhoods[vertex].size();
  return degree;
}



int BOOM::SparseGraph::getNumEdges() const
{
  return numEdges;
}



int BOOM::SparseGraph::getNumVertices() const
{
  return neighborhoods.size();
}



void BOOM::SparseGraph::addEdge(VertexId v1,VertexId v2)
{
  Neighborhood &v1neighbors=neighborhoods[v1];
  if(v1neighbors.find(v2)==v1neighbors.end())
    {
      v1neighbors.insert(v2);
      Neighborhood &v2neighbors=neighborhoods[v2];
      v2neighbors.insert(v1);
      ++numEdges;
    }
}



void BOOM::SparseGraph::removeEdge(VertexId v1,VertexId v2)
{
  Neighborhood &v1neighbors=neighborhoods[v1];
  Neighborhood::iterator cur=v1neighbors.find(v2);
  bool found=(cur!=v1neighbors.end());
  if(found)
    {
      v1neighbors.erase(cur);
      Neighborhood &v2neighbors=neighborhoods[v2];
      v2neighbors.erase(v2neighbors.find(v1));
      --numEdges;
    }
}
