/***********************************************************************
 Vector.H
 bmajoros@tigr.org

 BOOM : Bioinformatics Object Oriented Modules

 This software is open source, and is distributed under the terms of the
 ARTISTIC LICENSE, as described at www.opensource.org.
 ***********************************************************************/
#ifndef INCL_BOOM_Vector_H
#define INCL_BOOM_Vector_H
#include <iostream>
#include <vector>
using namespace std;

namespace BOOM {
template<class T>
class Vector : public vector<T>
{
public:
  Vector();
  Vector(int size);
  BOOM::Vector<T> *getSubrange(int from,int to);
  void getSubrange(int from,int to,BOOM::Vector<T> &);
  bool isMember(const T &);
  void append(const BOOM::Vector<T> &);
  void push_front(const T &);
  void removeFirst(const T &);
  bool isEmpty() const {return empty();}
  T getFirst() {return *begin();}
  T getLast() {return (*this)[size()-1];}
  void resize(int n) {vector<T>::resize(n);}
  void cut(int index);
  void cut(int startIndex,int howMany);
  void insertByIndex(const T &,int beforeIndex);
  void insertByIndex(const Vector<T> &,int beforeIndex);
  void reverse();
  inline void swap(int i,int j);
  void printOn(ostream &);
  void readFrom(istream &);
  void purge();
};

typedef BOOM::Vector<int> IntVector;
typedef BOOM::Vector<float> FloatVector;
typedef BOOM::Vector<double> DoubleVector;
typedef BOOM::Vector<char> CharVector;
typedef BOOM::Vector<long> LongVector;
}


template<class T>
BOOM::Vector<T>::Vector()
{
}



template<class T>
BOOM::Vector<T>::Vector(int size) : vector<T>(size)
{
}



template<class T>
BOOM::Vector<T> *BOOM::Vector<T>::getSubrange(int from,int to)
{
  BOOM::Vector<T> &newVector=*new BOOM::Vector<T>;
  getSubrange(from,to,newVector);
  return &newVector;
}



template<class T>
bool BOOM::Vector<T>::isMember(const T &x)
{
  typename BOOM::Vector<T>::iterator cur=begin(), end=end();
  for(; cur!=end ; ++cur)
    if(*cur==x) 
      return true;
  return false;
}



template<class T>
void BOOM::Vector<T>::append(const BOOM::Vector<T> &other)
{
  typename BOOM::Vector<T>::const_iterator cur=other.begin(), 
    end=other.end();
  for( ; cur!=end ; ++cur)
    push_back(*cur);
}



template<class T>
void BOOM::Vector<T>::getSubrange(int from,int to,BOOM::Vector<T> &into)
{
  BOOM::Vector<T> &self=*this;
  for(int i=from ; i<=to ; ++i)
    into.push_back(self[i]);
}



template<class T>
void BOOM::Vector<T>::push_front(const T &x)
{
  insert(begin(),x);
}



template<class T>
void BOOM::Vector<T>::removeFirst(const T &x)
{
  typename BOOM::Vector<T>::iterator cur=begin(), end=this->end();
  for(; cur!=end ; ++cur)
    if(*cur==x) 
      {
	erase(cur);
	return;
      }
}



template<class T>
void BOOM::Vector<T>::cut(int index)
{
  cut(index,1);
}



template<class T>
void BOOM::Vector<T>::cut(int i,int howMany)
{
  BOOM::Vector<T> &self=*this;
  int n=size();
  int src=i+howMany;
  for(; src<n ; ++i, ++src)
    self[i]=self[src];
  resize(n-howMany);
}



template<class T>
void BOOM::Vector<T>::insertByIndex(const T &newT,int beforeIndex)
{
  int n=size();
  BOOM::Vector<T> &self=*this;
  push_back(newT);
  for(int i=n ; i>beforeIndex ; --i) self[i]=self[i-1];
  self[beforeIndex]=newT;
}



template<class T>
void BOOM::Vector<T>::swap(int i,int j)
{
  BOOM::Vector<T> &self=*this;
  T t=self[i];
  self[i]=self[j];
  self[j]=t;
}



template<class T>
void BOOM::Vector<T>::reverse()
{
  for(int i=0, j=size()-1 ; i<j ; ++i, --j)
    swap(i,j);
}



template<class T>
void BOOM::Vector<T>::purge()
{
  clear();
}



template<class T>
void BOOM::Vector<T>::printOn(ostream &os)
{
  int n=size();
  os<<n<<endl;
  Vector<T> &self=*this;
  for(int i=0 ; i<n ; ++i)
    os<<self[i]<<'\t';
}



template<class T>
void BOOM::Vector<T>::readFrom(istream &is)
{
  int n;
  is>>n;
  resize(n);
  Vector<T> &self=*this;
  for(int i=0 ; i<n ; ++i)
    is>>self[i];
}



template<class T>
ostream &operator<<(ostream &os,BOOM::Vector<T> &v)
{
  v.printOn(os);
  return os;
}



template<class T>
istream &operator>>(istream &is,BOOM::Vector<T> &v)
{
  v.readFrom(is);
  return is;
}




template<class T>
void BOOM::Vector<T>::insertByIndex(const BOOM::Vector<T> &other,int pos)
{
  Vector<T> &self=*this;
  int L=size();
  int N=other.size();
  resize(L+N);
  int suffix=L-pos;
  for(int i=0 ; i<suffix ; ++i)
    self[L+N-1-i]=self[L-1-i];
  for(int i=0 ; i<N ; ++i)
    self[pos+i]=other[i];
}



#endif
