/****************************************************************
 test-threads.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <math.h>
#include "String.H"
#include "CommandLine.H"
#include "Thread.H"
#include "Time.H"
using namespace std;
using namespace BOOM;



class MyThread : public Thread
{
    double x;
public:
    MyThread() {}
    virtual void f()
        {
            double sum=0;
            for(int i=1 ; i<100000 ; ++i)
                sum+=log(double(i));
            x=sum;
        }
    double getX() {return x;}
};



class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
};



int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    CommandLine cmd(argc,argv,"u");
    if(cmd.numArgs()!=0)
      throw String("test-threads [-u]  (-u = uniprocessor mode)");
    bool uniprocessor=cmd.option('u');
    
    Time t;
    t.startCounting();
    MyThread t1;
    if(uniprocessor) t1.join();
    MyThread t2;
    if(!uniprocessor) t1.join();
    t2.join();
    t.stopCounting();
    cout<<t1.getX()<<"\t"<<t2.getX()<<"\tsec="<<t.elapsedSeconds()<<endl;
    return 0;
  }

