/****************************************************************
 TreeNode.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "TreeNode.H"
#include <iostream>


TreeNode::TreeNode(NodeType type)
  : type(type)
{
  // ctor
}



NodeType TreeNode::getType()
{
  return type;
}


// ====================== LeafNode =========================

LeafNode::LeafNode(int cat)
  : TreeNode(LEAF_NODE), category(cat)
{
  // ctor
}



int LeafNode::getCategory()
{
  return category;
}



// ===================PredicateNode=========================

PredicateNode::PredicateNode(int attributeNum,float value,float deltaH,
			     int mostPopularCategory,float originalH)
  : attributeNum(attributeNum), value(value),
    leftChild(NULL), rightChild(NULL), 
    TreeNode(PREDICATE_NODE), deltaH(deltaH),
    mostPopularCategory(mostPopularCategory),originalH(originalH)
{
  // ctor
}



PredicateNode::PredicateNode(int attributeNum,int value)
  : attributeNum(attributeNum), value((float)value),
    leftChild(NULL), rightChild(NULL),
    TreeNode(PREDICATE_NODE)
{
  // ctor
}



int PredicateNode::getMostPopularCategory()
{
  return mostPopularCategory;
}



void PredicateNode::attachLeftChild(TreeNode *left)
{
  leftChild=left;
}



void PredicateNode::attachRightChild(TreeNode *right)
{
  rightChild=right;
}



TreeNode *PredicateNode::getLeftChild()
{
  return leftChild;
}



TreeNode *PredicateNode::getRightChild()
{
  return rightChild;
}



int PredicateNode::getAttribute()
{
  return attributeNum;
}



float PredicateNode::getFloatValue()
{
  return value;
}



int PredicateNode::getIntValue()
{
  return int(value);
}



float PredicateNode::getDeltaH()
{
  return deltaH;
}



float PredicateNode::getOriginalH()
{
  return originalH;
}



// ======================TempNode===========================

TempNode::TempNode(BitSet *set,Distribution *distribution,
		   TreeNode *parent)
  : set(set), distribution(distribution),
    parent(parent), TreeNode(TEMP_NODE)
{
  // ctor
}



TempNode::~TempNode()
{
  delete set;
  delete distribution;
}



TreeNode *TempNode::getParent()
{
  return parent;
}



BitSet *TempNode::getSet()
{
  return set;
}



Distribution *TempNode::getDistribution()
{
  return distribution;
}





