/****************************************************************
 apply-tree.C
 william.majoros@duke.edu

 This program applies a decision tree (pruned or non-pruned) to
 a data set and writes out the predictions.

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#include <iostream>
#include <stdlib.h>
#include <fstream>
#include "BOOM/Exceptions.H"
#include "BOOM/String.H"
#include "BOOM/Stacktrace.H"
#include "BOOM/Time.H"
#include "BOOM/CommandLine.H"
#include "../common/NamesFile.H"
#include "../common/TestSet.H"
#include "../common/Schema.H"
#include "Classifier.H"



class Application
{
  NamesFile namesFile;
  Schema *schema;
  Classifier *classifier;
  String treeFile, nameFile, testFile, outfile;

  void runTestCases();
public:
  Application(String treeFile,String nameFile,String testFile,
	      String outfile);
  void go();
};


int main(int argc,char *argv[])
{
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=4)
    {
      cerr<<
"\napply-tree <tree-file> <names-file> <data-file> <output-file>\n"
	  <<endl;
      return -1;
    }

  try
    {
      Time stopwatch;
      stopwatch.startCounting();

      String treeFile=cmd.arg(0);
      String nameFile=cmd.arg(1);
      String testFile=cmd.arg(2);
      String outfile=cmd.arg(3);

      Application app(treeFile,nameFile,testFile,outfile);
      app.go();

      stopwatch.stopCounting();
      cout << stopwatch.elapsedTime() << endl;
    }
  catch(RootException &e)
    {
      cerr << e.getMessage() << endl;
      return 1;
    }
  catch(string s)
    {
      cout<<"exception: "<<s<<endl;
    }
  catch(const char *p)
    {
      cout<<"exception: "<<p<<endl;
    }
  catch(...)
    {
      cout<<"unknown exception caught in main()"<<endl;
    }

  return 0;
}



Application::Application::Application(String treeFile,String nameFile,
				      String testFile,String outfile)
  : treeFile(treeFile), nameFile(nameFile), testFile(testFile),
    outfile(outfile)
{
  // ctor
}



void Application::go()
{
  schema=namesFile.processFile(nameFile);
  DecisionTree tree(NULL);
  tree.load(treeFile);
  classifier=new Classifier(tree,*schema);
  runTestCases();
}



void Application::runTestCases()
{
  ofstream os(outfile.c_str());
  TestSet *testSet=new TestSet(*schema,testFile);
  int n=testSet->numCases();
  for(int i=0 ; i<n ; ++i)
    {
      TrainingCase &c=testSet->getCase(i);
      int predicted=classifier->classify(c);
      os<<predicted<<endl;
    }
}





