/****************************************************************
 print-tree.C
 william.majoros@duke.edu

 This program prints out a decision tree in a human-readable form.

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#include <iostream>
#include <stdlib.h>
#include <fstream>
#include "BOOM/Exceptions.H"
#include "BOOM/String.H"
#include "BOOM/Stacktrace.H"
#include "BOOM/CommandLine.H"
#include "../common/NamesFile.H"
#include "../common/TestSet.H"
#include "../common/Schema.H"
#include "TreeNode.H"
#include "DecisionTree.H"


class Application
{
  NamesFile namesFile;
  Schema *schema;
  String treeFile, nameFile;

  void printSubtree(TreeNode *,int numTabs,ostream &);
public:
  Application(String treeFile,String nameFile);
  void go();
};


int main(int argc,char *argv[])
{
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=2)
    {
      cerr<<"\napply-tree <tree-file> <names-file>\n"
	  <<endl;
      return -1;
    }

  try
    {
      String treeFile=cmd.arg(0);
      String nameFile=cmd.arg(1);
      Application app(treeFile,nameFile);
      app.go();
    }
  catch(RootException &e)
    {
      cerr << e.getMessage() << endl;
      return 1;
    }
  catch(string s)
    {
      cout<<"exception: "<<s<<endl;
    }
  catch(const char *p)
    {
      cout<<"exception: "<<p<<endl;
    }
  catch(...)
    {
      cout<<"unknown exception caught in main()"<<endl;
    }

  return 0;
}



Application::Application::Application(String treeFile,String nameFile)
  : treeFile(treeFile), nameFile(nameFile)
{
  // ctor
}



void Application::go()
{
  schema=namesFile.processFile(nameFile);
  DecisionTree tree(NULL);
  tree.load(treeFile);
  printSubtree(tree.getRoot(),0,cout);
}



void Application::printSubtree(TreeNode *node,int numTabs,
			       ostream &os)
{
  int i;
  for(i=0 ; i<numTabs ; ++i)
    os << "|   ";

  switch(node->getType())
    {
    case LEAF_NODE:
      {
	LeafNode *leafNode=(LeafNode *) node;
	os << "category=" << leafNode->getCategory() << endl;
      }
      break;
    case PREDICATE_NODE:
      {
	PredicateNode *predicateNode=(PredicateNode *) node;
	int attribute=predicateNode->getAttribute();
	String &name=namesFile.getName(attribute);
	os << name;
	switch((*schema)[attribute]->getType())
	  {
	  case DISCRETE:
	    os << "=" << predicateNode->getIntValue() << ":"
	       << endl;
	    break;
	  case CONTINUOUS:
	    os << "<=" << predicateNode->getFloatValue()
	       << ":" << endl;
	    break;
	  }
	printSubtree(predicateNode->getLeftChild(),
		     numTabs+1,os);
	printSubtree(predicateNode->getRightChild(),
		     numTabs+1,os);
      }
      break;
    case TEMP_NODE:
      throw Stacktrace("This shouldn't happen");
    }
}




