/****************************************************************
 Stats.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Stats.H"
#include <iostream>
#include "Tableau.H"
using namespace std;


Stats::Stats(Population &pop)
{
  // ctor

  computeStats(pop);
}



void Stats::computeStats(Population &pop)
{
  float heightSum=0.0;
  float sum=0.0;
  bool first=true;
  int i, n=pop.size();
  for(i=0 ; i<n ; ++i)
    {
      Individual *individual=pop.getMember(i);
      float fitness=individual->getStandardizedFitness();
      float accuracy=1-fitness;
      if(first)
	{
	  maxAccuracy=minAccuracy=accuracy;
	  first=false;
	}
      else if(accuracy>maxAccuracy) maxAccuracy=accuracy;
      else if(accuracy<minAccuracy) minAccuracy=accuracy;
      sum+=accuracy;
      heightSum+=individual->getCachedHeight();
    }
  aveAccuracy=sum/n;
  aveHeight=heightSum/n;
}



void Stats::printOn(ostream &os) const
{
  os.precision(3);
  os << "accuracy(" << minAccuracy << '-' << maxAccuracy
     << " av=" << aveAccuracy << ") ";
  if(Tableau::getGlobal().getMaxHeight()>0) 
    os << "av height=" << aveHeight;
}



ostream &operator<<(ostream &os,const Stats &stats)
{
  stats.printOn(os);
  return os;
}



