/****************************************************************
 SubtreeMutator.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "SubtreeMutator.H"
#include <iostream>
#include "BOOM/Random.H"
#include "Tableau.H"
using namespace std;
using namespace BOOM;



SubtreeMutator::SubtreeMutator(TreeBuilder &treeBuilder)
  : treeBuilder(treeBuilder),
    maxHeight(Tableau::getGlobal().getMaxHeight())
{
  // ctor
}



Individual *SubtreeMutator::mutate(Individual *individual)
{
  // First we clone the individual, so we can mutate the clone
  Individual *mutant=individual->clone();

  // Pick one tree (an ADF or an entry point) to mutate
  int treeNum=RandomNumber(mutant->getNumTrees());
  TreeNode *root=mutant->getTree(treeNum);

  // Pick a node in that tree to mutate
  vector<CrossoverPoint> &points=
    *crossPtCollector.collectPoints(root);
  int pointNum=RandomNumber(points.size());
  CrossoverPoint crossoverPoint=points[pointNum];
  delete &points;

  // Build a replacement subtree
  int depth=crossoverPoint.getDepth();
  int height=maxHeight-depth;
  if(height<1) height=1;
  height=RandomNumber(height)+1;
  TreeNode *newSubtree=treeBuilder.buildTree(height);
    
  // Install the new subtree in place of the old
  TreeNode *parent=crossoverPoint.getParent();
  TreeNode *oldSubtree=parent->getIthChild(crossoverPoint.getChildNum());
  parent->setIthChild(crossoverPoint.getChildNum(),newSubtree);

  // Delete the old subtree
  delete oldSubtree;

  return mutant;
}



