/****************************************************************
 Classifier.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Classifier_H
#define INCL_Classifier_H
#include <iostream>
#include "Model.H"
#include "../common/TrainingSet.H"
#include "BOOM/GSL/Vector.H"
using namespace std;
using namespace BOOM;


/****************************************************************
 A Classifier maps test cases to categories.  It does this using
 a Model and a set of centroid Vectors, via the nearest-centroid
 method.
 ****************************************************************/
class Classifier
{
public:
  Classifier();
  Classifier(Model *,TrainingSet &); // deletes Model but not TrainingSet
  virtual ~Classifier();
  int operator()(const Case &); // case -> category
  void printOn(ostream &) const;
  void readFrom(istream &);
  Model &getModel();
private:
  Model *model;
  BOOM::Vector<GSL::Vector*> centroids;
  void computeCentroids(TrainingSet &);
};

ostream &operator<<(ostream &,const Classifier &);
istream &operator>>(istream &,Classifier &);

#endif

