/****************************************************************
 Discriminant.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "Discriminant.H"
using namespace std;
using namespace BOOM;


Discriminant::Discriminant(GSL::Vector &eigenvector,double eigenvalue)
  : eigenvector(eigenvector),
    eigenvalue(eigenvalue)
{
  // ctor
}



double Discriminant::operator()(const Case &c) const
{
  // Compute a dot product ("inner product") between the eigenvector
  // and the case:
  int n=eigenvector.getDim();
  double sum=0;
  for(int i=0 ; i<n ; ++i)
    sum+=eigenvector[i]*c.getContinuous(i);
  return sum;
}



double Discriminant::getEigenvalue() const
{
  return eigenvalue;
}



void Discriminant::printOn(ostream &os) const
{
  os << eigenvalue << endl;
  os << eigenvector << endl;
}



void Discriminant::readFrom(istream &is)
{
  is>>eigenvalue>>eigenvector;
}



ostream &operator<<(ostream &os,const Discriminant &d)
{
  d.printOn(os);
  return os;
}



istream &operator>>(istream &is,Discriminant &d)
{
  d.readFrom(is);
  return is;
}
