/****************************************************************
 Model.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "Model.H"
using namespace std;
using namespace BOOM;



Model::~Model()
{
  // dtor

  BOOM::Vector<Discriminant*>::iterator cur=discriminants.begin(),
    end=discriminants.end();
  for(; cur!=end ; ++cur)
    delete *cur;
}



void Model::addDiscriminant(Discriminant *d)
{
  discriminants.push_back(d);
}



int Model::getDim() const
{
  return discriminants.size();
}



GSL::Vector *Model::operator()(const Case &c) const
{
  GSL::Vector *v=new GSL::Vector;
  mapToPoint(c,*v);
  return v;
}



void Model::mapToPoint(const Case &c,GSL::Vector &result) const
{
  int n=getDim();
  if(result.getDim()!=n) result.resize(n);
  for(int i=0 ; i<n ; ++i)
    result[i]=(*discriminants[i])(c);
}



void Model::printOn(ostream &os) const
{
  int n=getDim();
  os<<n<<endl;
  for(int i=0 ; i<n ; ++i)
    os<<*discriminants[i]<<endl;
}



void Model::readFrom(istream &is)
{
  discriminants.purge();
  int n;
  is>>n;
  for(int i=0 ; i<n ; ++i)
    {
      Discriminant *d=new Discriminant;
      is>>*d;
      discriminants.push_back(d);
    }
}



ostream &operator<<(ostream &os,const Model &m)
{
  m.printOn(os);
  return os;
}



istream &operator>>(istream &is,Model &m)
{
  m.readFrom(is);
  return is;
}


