/****************************************************************
 recode.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <math.h>
#include <fstream>
#include "BOOM/String.H"
#include "BOOM/CommandLine.H"
#include "BOOM/GSL/Matrix.H"
#include "BOOM/GSL/Vector.H"
#include "../common/Schema.H"
#include "../common/TrainingSet.H"
#include "../common/NamesFile.H"
#include "Discriminant.H"
#include "Model.H"
#include "Classifier.H"
using namespace std;
using namespace BOOM;


class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
{
  try
    {
      Application app;
      return app.main(argc,argv);
    }
  catch(const char *p)
    {
      cerr << p << endl;
    }
  catch(const string &msg)
    {
      cerr << msg.c_str() << endl;
    }
  catch(const exception &e)
    {
      cerr << "STL exception caught in main:\n" << e.what() << endl;
    }
  catch(...)
    {
      cerr << "Unknown exception caught in main" << endl;
    }
  return -1;
}



Application::Application()
{
  // ctor
}



int Application::main(int argc,char *argv[])
{
  // Process command line
  CommandLine cmd(argc,argv,"d:");
  if(cmd.numArgs()!=4) throw 
    "recode <model> <namesfile> <infile> <outfile>";
  String modelFile=cmd.arg(0);
  String namesFilename=cmd.arg(1);
  String infile=cmd.arg(2);
  String outfile=cmd.arg(3);
  
  // Load configuration
  NamesFile namesFile;
  Schema *schema=namesFile.processFile(namesFilename);
  
  // Load data
  TrainingSet testData(*schema,infile,false);
  int numCases=testData.numCases();
  
  // Load the model
  Classifier classifier;
  ifstream is(modelFile.c_str());
  if(!is.good()) throw String("can't open ")+modelFile;
  is>>classifier;
  is.close();
  Model &model=classifier.getModel();

  // Recode the data & produce output
  ofstream os(outfile.c_str());
  GSL::Vector v;
  for(int i=0 ; i<numCases ; ++i)
    {
      TrainingCase &c=testData.getCase(i);
      model.mapToPoint(c,v);
      int n=v.getDim();
      for(int j=0 ; j<n ; ++j)
	os<<v[j]<<"\t";
      os<<c.getCategory()<<endl;
    }
  
  return 0;
}



