/****************************************************************
 apply-components.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <fstream>
#include "BOOM/String.H"
#include "BOOM/CommandLine.H"
#include "BOOM/Vector.H"
#include "BOOM/GSL/Vector.H"
#include "../common/NamesFile.H"
#include "../common/TrainingSet.H"
using namespace std;
using namespace BOOM;


class Application
{
  NamesFile namesFile;
  BOOM::Vector<GSL::Vector*> model;
  GSL::Vector means, sigmas;
  void transformData(TrainingSet &in,TrainingSet &out,ostream &);
  void loadModel(const String &filename);
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
{
  // Process command line
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=4)
    throw String(
	 "apply-components <*.model> <*.names> <data-in> <data-out>");
  String modelFile=cmd.arg(0);
  String namesFilename=cmd.arg(1);
  String infile=cmd.arg(2);
  String outfile=cmd.arg(3);

  // Load schema
  Schema &schema=*namesFile.processFile(namesFilename);

  // Load model (components)
  loadModel(modelFile);
  
  // Load data
  TrainingSet trainingData(schema,infile);

  // Transform data
  ofstream os(outfile.c_str());
  TrainingSet transformedData(schema);
  transformData(trainingData,transformedData,os);

  return 0;
}



void Application::loadModel(const String &filename)
{
  ifstream is(filename.c_str());
  int n;
  is>>n;
  for(int i=0 ; i<n ; ++i)
    {
      GSL::Vector *v=new GSL::Vector;
      is>>*v;
      model.push_back(v);
    }
  is>>means>>sigmas;
}



void Application::transformData(TrainingSet &in,TrainingSet &out,
				ostream &os)
{
  int n=in.numCases(), m=model.size();
  for(int i=0 ; i<n ; ++i)
    {
      TrainingCase &c=in.getCase(i);
      for(int i=0 ; i<m ; ++i)
	{
	  GSL::Vector &coefficients=*model[i];
	  int dim=coefficients.getDim();
	  double sum=0;
	  for(int i=0 ; i<dim ; ++i)
	    sum+=coefficients[i]*(c.getContinuous(i)-means[i])/sigmas[i];
	  os<<sum<<"\t";
	}
      os<<c.getCategory()<<endl;
    }
}



