/****************************************************************
 Individual.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Individual_H
#define INCL_Individual_H

#include "BOOM/Vector.H"
#include "TreeNode.H"
#include "CrossoverPoint.H"
#include <iostream>
using namespace std;
using namespace BOOM;


class Individual
{
  Vector<TreeNode*> trees;
  int callCount; // for preventing infinite recursion
  float fitness;
  int height;

  TreeNode *loadTree(istream &);
  TreeNode *createNode(NodeType,int arity,istream &);
public:
  Individual(); // calls resetCallCount()
  ~Individual(); // deletes trees
  void decrementCallCount();
  bool canCall();
  void resetCallCount();
  TreeNode *&getADF(int adfNum);
  TreeNode *&getEntryPoint(int entryPointNum=0);
  int getNumTrees() const;
  TreeNode *getTree(int);
  Individual *clone() const;
  void printOn(ostream &) const;
  void save(ostream &) const;
  void load(istream &);
  int getTreeHeight();
  int getCachedHeight();

  inline float getFitness() { return fitness; }
  inline void setFitness(float f) {fitness=f;}
};

ostream &operator<<(ostream &,const Individual &);

#endif
