/****************************************************************
 Mutator.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Mutator.H"
#include <iostream>
#include "BOOM/Random.H"
using namespace std;
using namespace BOOM;


Mutator::Mutator(SymbolSet &symbolSet)
  : symbolSet(symbolSet)
{
  // ctor
}



Individual *Mutator::mutate(Individual *individual)
{
  // First we clone the individual, so we can mutate the clone
  Individual *mutant=individual->clone();

  // Pick one tree (an ADF or an entry point) to mutate
  int treeNum=RandomNumber(mutant->getNumTrees());
  TreeNode *root=mutant->getTree(treeNum);

  // Pick a node in that tree to mutate
  vector<CrossoverPoint> &points=
    *crossPtCollector.collectPoints(root);
  int pointNum=RandomNumber(points.size());
  CrossoverPoint crossoverPoint=points[pointNum];
  delete &points;

  // Determine whether the node is a terminal or a nonterminal
  TreeNode *parent=crossoverPoint.getParent();
  TreeNode *child=parent->getIthChild(crossoverPoint.getChildNum());
  const int numChildren=child->getNumChildren();
  bool terminal=numChildren==0;

  // Pick an appropriate replacement node
  TreeNode *newNode=
    terminal ? symbolSet.randomTerminal() : symbolSet.randomNonterminal();
    
  // Move the old node's children over to the new node
  int i;
  for(i=0 ; i<numChildren ; ++i)
    {
      newNode->addChild(child->getIthChild(i));
      child->setIthChild(i,NULL); // so it won't get deleted
    }

  // Install the new node in place of the old node
  parent->setIthChild(crossoverPoint.getChildNum(),newNode);

  // Delete the old node
  delete child;

  return mutant;
}



