/****************************************************************
 RouletteWheel.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/

#include "RouletteWheel.H"
#include "BOOM/Random.H"
#include <math.h>
#include <iostream>
using namespace std;
using namespace BOOM;



// **************************************************
//               RouletteWheel methods
// **************************************************

RouletteWheel::RouletteWheel(Population &population)
  : array(population.size()+1)
{
  // ctor

  initializeSectors(population);
}



RouletteWheel::RouletteWheel(vector<Individual*> &v,
			     int vectorSize)
  : array(vectorSize+1)
{
  initializeSectors(v,vectorSize);
}



int RouletteWheel::lookup(float f)
{
  int from=0, to=array.size()-1;
  
  while(from<to)
    {
      const int middle=(int) ceil((from+to)/2.0);
      if(f<array[middle])
	to=middle-1;
      else
	from=middle;
    }

  return from;
}



int RouletteWheel::spin()
{
  return lookup(Random0to1());
}



void RouletteWheel::initializeSectors(Population &population)
{
  // This method splits the wheel (all 360 degrees of it)
  // into sectors having arc-lengths proportional to
  // an individual's normalized fitness.

  float sectorEdge=0.0;

  int i, popSize=population.size();
  for(i=0 ; i<popSize ; ++i)
    {
      // Store the edge of this individual's sector
      array[i]=sectorEdge;

      const float arcLength=
	population.getMember(i)->getNormalizedFitness();

      sectorEdge+=arcLength;
    }

  array[i]=1.001;
}



void RouletteWheel::initializeSectors(vector<Individual*> &
				      population,
				      int popSize)
{
  // This method splits the wheel (all 360 degrees of it)
  // into sectors having arc-lengths proportional to
  // an individual's normalized fitness.

  float sectorEdge=0.0;
  int i;
  for(i=0 ; i<popSize ; ++i)
    {
      // Store the edge of this individual's sector
      array[i]=sectorEdge;

      const float arcLength=
	population[i]->getNormalizedFitness();

      sectorEdge+=arcLength;
    }

  array[i]=1.001;
}



