/****************************************************************
 Tableau.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Tableau_H
#define INCL_Tableau_H

#include "BOOM/ConfigFile.H"
using namespace BOOM;

class Tableau
{
  static Tableau *global;
  float Pm; // proportion of population produced using mutation
  float subtreeMutation; // how much of pop. comes from subtree mutation
  float minConst; // min value of a constant node
  float maxConst; // max value of a constant node
  int initialHeight; // max tree height of intial population
  int maxHeight; // max height for any tree, any population
  int seed; // seed for randomizer, or 0 if none
  String nonterminals; // comma-sep. list of nonterminals (operators)
  String terminals; // comma-sep. list of terminals (leaves)
  float trainPortion; // how much of the training set each individual
                      // is exercised on during training
  float initialTemperature;
  float finalTemperature;
  float temperatureDecay;
  double K;

  Tableau(ConfigFile &);
public:
  static void processConfig(ConfigFile &);
  static Tableau &getGlobal();
  float getPm() {return Pm;}
  float getSubtreeMutation() {return subtreeMutation;}
  float getMinConst() {return minConst;}
  float getMaxConst() {return maxConst;}
  int getInitialHeight() {return initialHeight;}
  int getMaxHeight() {return maxHeight;}
  int getSeed() {return seed;}
  String getNonterminals() {return nonterminals;}
  String getTerminals() {return terminals;}
  float getTrainPortion() {return trainPortion;}
  float getInitialTemperature() {return initialTemperature;}
  float getFinalTemperature() {return finalTemperature;}
  float getTemperatureDecay() {return temperatureDecay;}
  double getK() {return K;}
};


#endif
