/****************************************************************
 BayesNetClassifier.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <math.h>
#include "BayesNetClassifier.H"
using namespace std;
using namespace BOOM;



BayesNetClassifier::BayesNetClassifier(int numVariables,int numCategories)
  : priors(numCategories),
    networks(numCategories)
{
  networks.setAllTo(NULL);
}



BayesNetClassifier::BayesNetClassifier(istream &is)
{
  load(is);
}



BayesNetClassifier::~BayesNetClassifier()
{
  int n=numCategories();
  for(int i=0 ; i<n ; ++i)
    delete networks[i];
}



void BayesNetClassifier::save(ostream &os)
{
  int n=numCategories();
  os<<n<<endl<<priors<<endl;
  for(int i=0 ; i<n ; ++i)
    os<<*networks[i]<<endl;
}



void BayesNetClassifier::load(istream &is)
{
  int n;
  is>>n;
  priors.resize(n);
  networks.resize(n);
  is>>priors;
  for(int i=0 ; i<n ; ++i)
    networks[i]=new BayesNetwork(is);
}



int BayesNetClassifier::classify(Case &c)
{
  double bestP;
  int n=numCategories(), cat=-1;
  for(int i=0 ; i<n ; ++i)
    {
      double P=networks[i]->P(c) + log(priors[i]);
      if(cat<0 || P>bestP)
	{
	  bestP=P;
	  cat=i;
	}
    }
  return cat;
}



void BayesNetClassifier::estimatePriors(TrainingSet &T)
{
  int n=numCategories();
  Array1D<int> counts(n);
  counts.setAllTo(0);
  int m=T.numCases();
  for(int i=0 ; i<m ; ++i)
    ++counts[T.getCase(i).getCategory()];
  double dm=m;
  for(int i=0 ; i<n ; ++i)
    priors[i]=counts[i]/dm;
}



void BayesNetClassifier::setNetwork(int category,BayesNetwork *net)
{
  networks[category]=net;
}


