/****************************************************************
 BayesNetClassifier.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_BayesNetClassifier_H
#define INCL_BayesNetClassifier_H
#include <iostream>
#include "../common/TrainingSet.H"
#include "BOOM/Array1D.H"
#include "BayesNetwork.H"
using namespace std;
using namespace BOOM;


/*******************************************************************
 A classifier that computes:

                      Y* = argmax_Y P(X|Y)*P(Y)

 for class Y and test case X=(X0,X1,...,Xn) for predictor variables
 Xi.  In Naive Bayes we would assume conditional independence and
 compute P(X|Y)=P(X0|Y)*...*P(Xn|Y).  Here we instead use a Bayes
 net to decompose the P(X|Y) term into a product of conditional
 probabilities: P(X|Y)=P(X0|parent(X0),Y)*...*P(Xn|parent(Xn),Y),
 where the parents are determined by computing a minimal spanning
 tree with edge weights given by Chi-squared independence statistics.
********************************************************************/
class BayesNetClassifier
{
public:
  BayesNetClassifier(int numVariables,int numCategories);
  BayesNetClassifier(istream &);
  virtual ~BayesNetClassifier();
  void save(ostream &);
  void load(istream &);
  int classify(Case &);
  void estimatePriors(TrainingSet &);
  void setNetwork(int category,BayesNetwork *);
  inline int numCategories() {return priors.size();}
private:
  Array1D<double> priors; // P(Y)
  Array1D<BayesNetwork*> networks; // one per category; computes P(X|Y)
};

#endif

