/****************************************************************
 BayesNetwork.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <math.h>
#include "BayesNetwork.H"
using namespace std;
using namespace BOOM;

BayesNetwork::BayesNetwork(Array1D<Discretizer*> &discretizers)
  : parentIndex(discretizers.size()),
    probTables(discretizers.size()),
    discretizers(discretizers)
{
  parentIndex.setAllTo(-1);
}



BayesNetwork::BayesNetwork(istream &is)
{
  load(is);
}



BayesNetwork::~BayesNetwork()
{
  int n=discretizers.size();
  for(int i=0 ; i<n ; ++i)
    delete discretizers[i];
}



double BayesNetwork::P(Case &c)
{
  double P=0.0;
  int n=discretizers.size();
  for(int i=0 ; i<n ; ++i)
    {
      int parent=parentIndex[i];
      Discretizer *childDiscretizer=discretizers[i];
      Discretizer *parentDiscretizer=discretizers[parent];
      int childVal=Discretizer::getDiscrete(c,i,childDiscretizer);
      double thisP;
      if(parent<0)
	thisP=probTables[i][childVal][0];
      else
	{
	  ConditionalDensity &density=probTables[i];
	  int parentVal=
	    Discretizer::getDiscrete(c,parent,parentDiscretizer);
	  thisP=density[childVal][parentVal];
	}
      P+=log(thisP);
    }
  return P;
}



void BayesNetwork::save(ostream &os)
{
  os<<parentIndex<<endl;
  os<<probTables<<endl;
  int n=discretizers.size();
  for(int i=0 ; i<n ; ++i)
    os<<*discretizers[i]<<endl;
}



void BayesNetwork::load(istream &is)
{
  int n=discretizers.size();
  for(int i=0 ; i<n ; ++i)
    delete discretizers[i];

  is>>parentIndex>>probTables;
  n=parentIndex.size();
  discretizers.resize(n);
  for(int i=0 ; i<n ; ++i)
    discretizers[i]=new Discretizer(is);
}



void BayesNetwork::addEdge(int parent,int child)
{
  parentIndex[child]=parent;
}



ConditionalDensity &BayesNetwork::getDensity(int child)
{
  return probTables[child];
}



ostream &operator<<(ostream &os,BayesNetwork &net)
{
  net.save(os);
  return os;
}



istream &operator>>(istream &is,BayesNetwork &net)
{
  net.load(is);
  return is;
}


