/****************************************************************
 BayesNetwork.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_BayesNetwork_H
#define INCL_BayesNetwork_H
#include <iostream>
#include "../common/Case.H"
#include "BOOM/Array1D.H"
#include "BOOM/Array2D.H"
#include "Discretizer.H"
using namespace std;
using namespace BOOM;

typedef DblArray2D ConditionalDensity;

/*******************************************************************
 This class computes:

                               P(X|Y)

 for *fixed* class Y and test case X=(X0,X1,...,Xn) for predictor 
 variables Xi.  We decompose P(X|Y) into a product of conditional
 probabilities: 

       P(X|Y) = P(X0|parent(X0),Y) * ... * P(Xn|parent(Xn),Y)

 where the parents are determined by computing a minimal spanning
 tree with edge weights given by Chi-squared independence statistics.
 Note that you must have one BayesNetwork per category.
********************************************************************/
class BayesNetwork
{
public:
  BayesNetwork(Array1D<Discretizer*> &d); // d will be deleted
  BayesNetwork(istream &);
  virtual ~BayesNetwork();
  double P(Case &);  // P(X|Y) where Y is fixed for each network
  void save(ostream &);
  void load(istream &);
  void addEdge(int parent,int child);
  ConditionalDensity &getDensity(int child);
private:
  Array1D<int> parentIndex; // parent(Xi)=Xj for j=parentIndex[i] (or -1)
  Array1D<ConditionalDensity> probTables; // P(Xi|Xj)=probTables[i][Xi][Xj]
  Array1D<Discretizer*> discretizers;
};

ostream &operator<<(ostream &,BayesNetwork &);
istream &operator>>(istream &,BayesNetwork &);

#endif

