/****************************************************************
 Discretizer.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Discretizer.H"


Discretizer::Discretizer(istream &is)
{
  // ctor

  load(is);
}



Discretizer::Discretizer(float minValue,float maxValue,float bucketSize,
			 int numBuckets)
  : minValue(minValue), 
    maxValue(maxValue),
    bucketSize(bucketSize),
    numBuckets(numBuckets)
{
  // ctor
}



int Discretizer::discretize(float value)
{
 int d=int((value-minValue)/bucketSize);
 if(d<0) d=0;
 else if(d>=numBuckets) d=numBuckets-1;
 return d;
}



void Discretizer::load(istream &is)
{
  is>>minValue>>maxValue>>bucketSize>>numBuckets;
}



void Discretizer::printOn(ostream &os)
{
  os<<minValue<<"\t"<<maxValue<<'\t'<<bucketSize<<'\t'<<numBuckets<<endl;
}



ostream &operator<<(ostream &os,Discretizer &d)
{
  d.printOn(os);
  return os;
}



istream &operator>>(istream &is,Discretizer &d)
{
  d.load(is);
  return is;
}



int Discretizer::getNumBuckets() const 
{
  return numBuckets;
}



float Discretizer::getMinValue() const 
{
  return minValue;
}



float Discretizer::getMaxValue() const 
{
  return maxValue;
}



float Discretizer::getBucketSize() const 
{
  return bucketSize;
}



int Discretizer::getDiscrete(Case &c,int attr,Discretizer *discretizer)
{
  return 
    discretizer 
    ?
    discretizer->discretize(c.getContinuous(attr))
    :
    c.getDiscrete(attr);
}
