/****************************************************************
 apply-bayes-net.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <fstream>
#include "BOOM/String.H"
#include "BOOM/CommandLine.H"
#include "../common/Schema.H"
#include "../common/TrainingSet.H"
#include "../common/NamesFile.H"
#include "BayesNetClassifier.H"
using namespace std;
using namespace BOOM;


class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
private:
  NamesFile namesFile;
  Schema *schema;
  int numCases, numAttributes, numCategories;
  TrainingSet *testData;
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    CommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=4)
      throw String(
"apply-bayes-net <*.model> <*.names> <*.data> <outfile>");
    String modelFile=cmd.arg(0);
    String namesFilename=cmd.arg(1);
    String dataFile=cmd.arg(2);
    String outfile=cmd.arg(3);
    
    // Load configuration
    schema=namesFile.processFile(namesFilename);
    numAttributes=schema->numAttributes();
    numCategories=schema->getNumCategories();

    // Load test data
    testData=new TrainingSet(*schema,dataFile,false);
    numCases=testData->numCases();
    
    // Load model
    ifstream is(modelFile.c_str());
    BayesNetClassifier classifier(is);
    is.close();
    
    // Generate output
    ofstream os(outfile.c_str());
    for(int i=0 ; i<numCases ; ++i)
      {
	Case &c=testData->getCase(i);
	os<<classifier.classify(c)<<endl;
      }
    os.close();
    
    return 0;
  }




