/****************************************************************
 Model.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Model.H"
using namespace BOOM;


Model::Model(PublicModel &p,Schema &s)
  : publicModel(p), schema(s)
{
  // ctor
}



float Model::P_xi_given_y(int categoryIndex,Case &thisCase)
{
  float p=P_y_given_xi(thisCase,categoryIndex) * P_xi(categoryIndex);
  return p;
}



float Model::P_y_given_xi(Case &thisCase,int categoryIndex)
{
  //const int categoryIndex=publicModel.x2index(category);

  const int n=publicModel.yStructure.getNumPredictors();
  float p=1.0;
  int i;
  for(i=0 ; i<n ; ++i)
    {
      int discreteValue;
      switch(schema[i]->getType())
	{
	case CONTINUOUS:
	  {
	    Discretizer &d=publicModel.yStructure[i].getDiscretizer();
	    float value=thisCase.getContinuous(i);
	    discreteValue=d.discretize(value);
	    break;
	  }
	case DISCRETE:
	  discreteValue=thisCase.getDiscrete(i);
	  break;
	}

      p*=publicModel.yStructure.lookup(i,discreteValue,categoryIndex);
    }

  return p;
}



float Model::P_xi(int categoryIndex)
{
  //const int categoryIndex=publicModel.x2index(category);
  float proportion=publicModel.xProportions[categoryIndex];
  return proportion;
}



int Model::operator()(Case &thisCase)
{
  int bestCategory=-1;
  float bestP=-1.0;

  const int numCategories=publicModel.xProportions.size();
  int i;
  for(i=0 ; i<numCategories ; ++i)
    {
      const int categoryIndex=i;
      float p=P_xi_given_y(categoryIndex,thisCase);
      
      if(p>bestP)
	{
	  bestCategory=categoryIndex;
	  bestP=p;
	}
    }

  return publicModel.index2x[bestCategory];
}



void Model::save(const String &filename)
{
  publicModel.save(filename);
}



Model *Model::load(const String &filename,Schema &s)
{
  PublicModel *p=new PublicModel(filename);
  return new Model(*p,s);
}



