/****************************************************************
 PublicModel.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "PublicModel.H"
#include <fstream>
using namespace std;
using namespace BOOM;


PublicModel::PublicModel(const String &filename)
  : x2index(*new Map<int,int>)
{
  // ctor

  load(filename);
}



PublicModel::PublicModel(int numPredictors,int numCategories,
			 Map<int,int> &x2index)
  : yStructure(numPredictors,numCategories),
    xProportions(numCategories),
    x2index(x2index)
{
  // ctor

  xProportions.setAllTo(0.0);

  // Invert the map
  Map<int,int>::iterator cur=x2index.begin(), end=x2index.end();
  for(; cur!=end ; ++cur)
    index2x[(*cur).second]=(*cur).first;
}



void PublicModel::save(const String &filename)
{
  ofstream os(filename.c_str());
  yStructure.save(os);
  os<<endl<<xProportions<<endl;
  os<<x2index<<endl;
  os<<index2x<<endl;
}



void PublicModel::load(const String &filename)
{
  ifstream is(filename.c_str());
  yStructure.load(is);
  is>>xProportions;
  is>>x2index;
  is>>index2x;
}


