/****************************************************************
 YStructure.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "YStructure.H"
using namespace BOOM;

YStructure::YStructure(int numPredictors,int numCategories)
  : array(numPredictors)
{
  for(int i=0 ; i<numPredictors ; ++i)
    array[i]=new YStructureElement(numCategories);
}



float &YStructure::lookup(int whichPredictor,int predictorValue,
			  int categoryIndex)
{
  YStructureElement *elem=array[whichPredictor];
  return elem->probability(predictorValue,categoryIndex);
}



void YStructure::allocateDiscretizer(int whichPredictor,float minValue,
				     float bucketSize)
{
  YStructureElement *elem=array[whichPredictor];
  elem->allocateDiscretizer(minValue,bucketSize);
}



YStructureElement &YStructure::operator[](int i)
{
  return *array[i];
}



int YStructure::getNumPredictors() const
{
  return array.size();
}



void YStructure::load(istream &is)
{
  int numPredictors;
  is>>numCategories>>numPredictors;
  array.resize(numPredictors);
  for(int i=0 ; i<numPredictors ; ++i)
    array[i]=new YStructureElement(is);
}



void YStructure::save(ostream &os)
{
  int numPredictors=array.size();
  os<<numCategories<<'\t'<<numPredictors<<endl;
  for(int i=0 ; i<numPredictors ; ++i)
    {
      YStructureElement *elem=array[i];
      os<<*elem<<endl;
    }
}



