/****************************************************************
 YStructureElement.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "YStructureElement.H"
#include "BOOM/Array1D.H"
using namespace BOOM;


YStructureElement::YStructureElement(istream &is)
{
  // ctor

  load(is);
}



YStructureElement::YStructureElement(int numCategories)
  : discretizer(NULL), 
    numCategories(numCategories)
{
  // ctor
}



float &YStructureElement::probability(int predictorValue,
				      int categoryIndex)
{
  if(!assocArray.isDefined(predictorValue))
    {
      FloatArray1D *newArray=new FloatArray1D(numCategories);
      newArray->setAllTo(0.0);	
      assocArray[predictorValue]=newArray;
    }

  FloatArray1D &array=*assocArray[predictorValue];
  return array[categoryIndex];
}



Discretizer &YStructureElement::getDiscretizer()
{
  if(!discretizer)
    throw "requesting null Discretizer from YStructureElement";
  return *discretizer;
}



void YStructureElement::allocateDiscretizer(float minValue,
					    float bucketSize)
{
  discretizer=new Discretizer(minValue,bucketSize);
}



Map<int,FloatArray1D*>::iterator YStructureElement::getIterator()
{
  return assocArray.begin();
}



Map<int,FloatArray1D*>::iterator YStructureElement::getIteratorEnd()
{
  return assocArray.end();
}



ostream &operator<<(ostream &os,YStructureElement &e)
{
  e.printOn(os);
  return os;
}



void YStructureElement::printOn(ostream &os)
{
  os<<numCategories<<endl;
  os<<*discretizer<<endl;
  int n=assocArray.numElements();
  os<<n<<endl;
  Map<int,FloatArray1D*>::iterator cur=assocArray.begin(), 
    end=assocArray.end();
  for(; cur!=end ; ++cur)
    {
      pair<const int,FloatArray1D*> &p=*cur;
      os<<p.first<<endl;
      os<<*p.second<<endl;
    }
}



void YStructureElement::load(istream &is)
{
  is>>numCategories;
  discretizer=new Discretizer(is);
  int n;
  is>>n;
  int first;
  FloatArray1D *second;
  for(int i=0 ; i<n ; ++i)
    {
      is>>first;
      second=new FloatArray1D;
      is>>*second;
      assocArray[first]=second;
    }
}


