/****************************************************************
 apply-bayes.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <fstream>
#include "BOOM/Exceptions.H"
#include "BOOM/CommandLine.H"
#include "../common/Schema.H"
#include "../common/TrainingSet.H"
#include "../common/NamesFile.H"
#include "ModelBuilder.H"
using namespace std;
using namespace BOOM;

// ######################################################

class Application
{
public:
  void go(int argc,char *argv[]);
  void predict(Model &,TrainingSet &,String &outfile);
};

// ######################################################

int main(int argc,char *argv[])
{
  try
    {
      Application app;
      app.go(argc,argv);
    }
  catch(RootException &e)
    {
      cerr << e.getMessage() << endl;
      return -1;
    }
  catch(const String &s)
    {
      cerr << s << endl;
      return -1;
    }
  catch(const char *p)
    {
      cerr<<p<<endl;
      return -1;
    }
  catch(...)
    {
      cerr << "Unknown exception caught in main()" << endl;
      return -1;
    }
  return 0;
}

// ######################################################

void Application::go(int argc,char *argv[])
{
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=4)
    throw "apply-bayes <model-file> <names-file> <test-file> <out-file>";
  String modelFile=cmd.arg(0);
  String namesFilename=cmd.arg(1);
  String testFile=cmd.arg(2);
  String outfile=cmd.arg(3);

  // Load configuration
  NamesFile namesFile;
  Schema &schema=*namesFile.processFile(namesFilename);

  // Load model
  Model *model=Model::load(modelFile,schema);

  // Load test data
  TrainingSet testData(schema,testFile);

  // Make predictions
  predict(*model,testData,outfile);
}



void Application::predict(Model &model,TrainingSet &dataSet,
			  String &outfile)
{
  ofstream os(outfile.c_str());
  int n=dataSet.numCases();
  for(int i=0 ; i<n ; ++i)
    {
      TrainingCase &thisCase=dataSet.getCase(i);
      const int predictedValue=model(thisCase);
      os<<predictedValue<<endl;
    }
}


