/****************************************************************
 train-bayes.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "BOOM/Exceptions.H"
#include "BOOM/CommandLine.H"
#include "../common/Schema.H"
#include "../common/TrainingSet.H"
#include "../common/NamesFile.H"
#include "ModelBuilder.H"
using namespace std;
using namespace BOOM;

int NUM_BUCKETS; // for discretization of continuous attributes


// ######################################################

class Application
{
public:
  void go(int argc,char *argv[]);
  void validate(Model &,TrainingSet &,const char *label,
		int &TP,int &TN,int &FP,int &FN);
};

// ######################################################

int main(int argc,char *argv[])
{
  try
    {
      Application app;
      app.go(argc,argv);
    }
  catch(RootException &e)
    {
      cerr << e.getMessage() << endl;
      return -1;
    }
  catch(const String &s)
    {
      cerr << s << endl;
      return -1;
    }
  catch(const char *p)
    {
      cerr<<p<<endl;
      return -1;
    }
  catch(...)
    {
      cerr << "Unknown exception caught in main()" << endl;
      return -1;
    }
  return 0;
}

// ######################################################

void Application::go(int argc,char *argv[])
{
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=4) throw 
"train-bayes <train-file> <names-file> <out-file> <#discretization-bins>";
  String trainFile=cmd.arg(0);
  String namesFilename=cmd.arg(1);
  String outfile=cmd.arg(2);
  NUM_BUCKETS=cmd.arg(3).asInt();

  // Load configuration
  NamesFile namesFile;
  Schema &schema=*namesFile.processFile(namesFilename);

  // Load training data
  TrainingSet trainingData(schema,trainFile);

  // Build model
  ModelBuilder modelBuilder(trainingData,schema);
  Model *model=modelBuilder.buildModel();

  // Save model into file
  model->save(outfile);
}



