/****************************************************************
 AttributeIterator.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "AttributeIterator.H"
#include <iostream>
using namespace BOOM;


AttributeIterator::AttributeIterator(AttrRangeTable &t)
  : schema(t.getSchema()), arrays(t.getArrays())
{
  // ctor

  reset();
}



void AttributeIterator::reset()
{
  currentAttribute=0;
  currentArray=arrays[0];
  currentArrayIndex=0;
  done=false;
}



bool AttributeIterator::hasMoreElements()
{
  return !done;
}



AttrValuePair AttributeIterator::currentElement()
{
  switch(schema[currentAttribute]->getType())
    {
    case DISCRETE:
      {
	IntArray &intArray=static_cast<IntArray&>(*currentArray);
	return AttrValuePair(currentAttribute,
			     intArray.values[currentArrayIndex]);
      }
      break;
    case CONTINUOUS:
      {
	DoubleArray &doubleArray=static_cast<DoubleArray&>(*currentArray);
	double mid, f1=doubleArray.values[currentArrayIndex];
	if(doubleArray.getSize()==1)
	  {
	    mid=f1;
	    cerr << "WARNING: Attribute " << currentAttribute
		 << " has only one unique value" << endl;
	  }
	else
	  {
	    double f2=doubleArray.values[currentArrayIndex+1];
	    mid=(f1+f2)/2;
	  }
	return AttrValuePair(currentAttribute,mid);
      }
      break;
    }
}



void AttributeIterator::advance()
{
  ++currentArrayIndex;

  bool nextArray=false;
  switch(schema[currentAttribute]->getType())
    {
    case DISCRETE:
      if(currentArrayIndex>=currentArray->getSize())
	nextArray=true;
      break;
    case CONTINUOUS:
      if(currentArrayIndex+1>=currentArray->getSize())
	nextArray=true;
      break;
    }

  if(nextArray)
    {
      ++currentAttribute;
      if(currentAttribute>=arrays.size())
	{
	  done=true;
	  return;
	}
      currentArray=arrays[currentAttribute];
      currentArrayIndex=0;
    }
}




