/****************************************************************
 NamesFile.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "NamesFile.H"
#include <iostream>
#include <fstream>
#include "BOOM/StrTokenizer.H"
using namespace BOOM;


NamesFile::NamesFile()
{
  // ctor
}



Schema *NamesFile::processFile(const String &filename)
{
  ifstream is(filename.c_str());
  if(!is.good())
    throw String("Cannot open names file: ")+filename;
  
  // Process first line (number of categories)
  String line;
  getline(is,line);
  if(line.length()==0)
    throw String("Syntax error in names file");
  StrTokenizer tokenizer(line.c_str()," \t\n\r.:");
  String numString=tokenizer.nextToken();
  int numCategories=numString.asInt();
  if(numCategories<=0)
    throw String("bad number of categories in names file");
  String cat=tokenizer.nextToken();
  if(!(cat=="categories"))
    throw String("Syntax error in names file - ")+
      "expecting keyword \"categories\"";

  Schema *schema=new Schema(numCategories);

  // Process subsequent lines
  while(!is.eof())
    {
      getline(is,line);
      if(line.length()==0) break;

      StrTokenizer tokenizer(line.c_str()," \t\n\r.:");
      while(tokenizer.hasMoreTokens())
	{
	  // Read the next line
	  String attribute=tokenizer.nextToken();
	  String typeString=tokenizer.nextToken();
	  
	  // Allocate a type descriptor
	  TypeDescriptor *descriptor=NULL;
	  if(typeString=="discrete")
	    descriptor=new Discrete(0,attribute);
	  else if(typeString=="continuous")
	    descriptor=new Continuous(attribute);
	  else
	    throw String("Illegal attribute type in names file");
	  
	  // Add the type descriptor to the schema
	  schema->add(descriptor);

	  // Store the name
	  names.push_back(attribute);
	}
    }

  return schema;
}



String &NamesFile::getName(int i)
{
  return names[i];
}



void NamesFile::addName(const String &name)
{
  names.push_back(name);
}


