/****************************************************************
 Schema.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "Schema.H"
#include "BOOM/Array1D.H"
#include "TrainingSet.H"
using namespace BOOM;


Schema::Schema(int num)
  : numCategories(num)
{
  // ctor
}



void Schema::add(TypeDescriptor *d)
{
  v.push_back(d);
}



int Schema::numAttributes()
{
  return v.size();
}



TypeDescriptor *Schema::operator[](int i)
{
  return v[i];
}



int Schema::getNumCategories()
{
  return numCategories;
}



void Schema::removeAttribute(int i)
{
  delete v[i];
  v.cut(i);
}



void Schema::printOn(ostream &os) const
{
  os << numCategories << " categories" << endl;

  int i, n=v.size();
  for(i=0 ; i<n ; ++i)
    os << *v[i] << endl;
}



ostream &operator<<(ostream &os,const Schema &schema)
{
  schema.printOn(os);
  return os;
}



Map<int,int> *Schema::countDiscrete(TrainingSet &set)
{
  // First, get indices of discrete attributes
  Vector<int> indices;
  int numAttr=numAttributes();
  for(int i=0 ; i<numAttr ; ++i)
    if(v[i]->getType()==DISCRETE) 
      indices.push_back(i);
  int numDiscreteAttr=indices.size();

  // Iterate through the cases, noting each unique
  // value of each discrete attribute
  Map<int,int> *maps=new Map<int,int>[numDiscreteAttr];
  int numCases=set.numCases();
  Array1D<int> nextLevel(numDiscreteAttr);
  nextLevel.setAllTo(0);
  for(int i=0 ; i<numCases ; ++i)
    {
      TrainingCase &c=set.getCase(i);
      for(int j=0 ; j<numDiscreteAttr ; ++j)
	{
	  int attr=indices[j];
	  int level=c.getDiscrete(attr);
	  if(maps[j].find(level)==maps[j].end())
	    maps[j][level]=nextLevel[j]++;
	}
    }

  // Store counts in individual TypeDescriptors
  for(int i=0 ; i<numDiscreteAttr ; ++i)
    {
      int index=indices[i];
      Discrete &discrete=(Discrete&)*v[index];
      discrete.setN(maps[i].size());
    }

  return maps;
}


