/****************************************************************
 Classifier.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Classifier_H
#define INCL_Classifier_H
#include <iostream>
#include "LogisticFunction.H"
#include "BOOM/Vector.H"
using namespace std;
using namespace BOOM;


enum ClassifierType
  {
    CL_BINARY,
    CL_MULTICLASS
  };


/****************************************************************
                       Abstract base class
 */
struct Classifier
{
  virtual int classify(const Case &)=0;
  virtual void save(ostream &)=0;
  static Classifier *load(istream &);
};




/****************************************************************
        The standard logistic regression binary classifier
 */
struct BinaryClassifier : public Classifier
{
  BinaryClassifier(const LogisticFunction &); // f is for category 0!
  BinaryClassifier(istream &);
  virtual int classify(const Case &);
  virtual void save(ostream &);
private:
  LogisticFunction f; // f is for category zero!
};



/****************************************************************
          Multi-class classification by max probability
 */
struct MulticlassClassifier : public Classifier
{
  MulticlassClassifier();
  MulticlassClassifier(istream &);
  void addFunction(const LogisticFunction &);
  virtual int classify(const Case &);
  virtual void save(ostream &);
private:
  BOOM::Vector<LogisticFunction> f;
};

#endif

