/****************************************************************
 LogLikelihood.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "LogLikelihood.H"
#include "BOOM/Constants.H"
#include <gsl/gsl_errno.h>
using namespace std;
using namespace BOOM;


LogLikelihood::LogLikelihood(TrainingSet &positives,TrainingSet &negatives)
  : positives(positives),
    negatives(negatives),
    numPos(positives.numCases()),
    numNeg(negatives.numCases()),
    numAttributes(positives.getSchema().numAttributes()),
    func(positives.getSchema().numAttributes()),
    ok(true)
{
}



void LogLikelihood::initFunc(LogisticFunction &func,
			     const GSL::Vector &currentPoint)
{
  func.getA()=currentPoint[0];
  for(int i=0 ; i<numAttributes ; ++i)
    func.getB(i)=currentPoint[i+1];
}



double LogLikelihood::f(const GSL::Vector &currentPoint)
{
  initFunc(func,currentPoint);
  
  double sum=0;
  for(int i=0 ; i<numPos ; ++i)
    {
      TrainingCase &c=positives.getCase(i);
      sum+=log(func(c));
    }
  for(int i=0 ; i<numNeg ; ++i)
    {
      TrainingCase &c=negatives.getCase(i);
      sum+=log(1-func(c));
    }
  return -sum; // negative, because the optimizer minimizes this function
}



void LogLikelihood::gradient(const GSL::Vector &currentPoint,
			     GSL::Vector &gradient)
{
  for(int i=-1 ; i<numAttributes ; ++i)
    {
      double sum=0;
      for(int j=0 ; j<numPos ; ++j)
	{
	  TrainingCase &c=positives.getCase(j);
	  double xi=(i>=0?c.getContinuous(i):1.0);
	  double lambda=func.lambda(c);
	  double PX=1/(1+lambda);
	  sum+=xi*lambda/(sqr(1+lambda)*PX);
	}
      for(int j=0 ; j<numNeg ; ++j)
	{
	  TrainingCase &c=negatives.getCase(j);
	  double xi=(i>=0?c.getContinuous(i):1.0);
	  double lambda=func.lambda(c);
	  double PX=1/(1+lambda);
	  sum-=xi*lambda/(sqr(1+lambda)*(1-PX));
	}
      gradient[i+1]=-sum; // minus because we're maximizing, not minimizing
      if(!isFinite(sum)) 
	{
	  ok=false;
	  gradient.setAllTo(0.0); // force the optimizer to quit
	  return;
	}
    }
}


