/****************************************************************
 LogisticFunction.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_LogisticFunction_H
#define INCL_LogisticFunction_H
#include <iostream>
#include <math.h>
#include "BOOM/Array1D.H"
#include "../common/TrainingCase.H"
using namespace std;
using namespace BOOM;


class LogisticFunction
{
public:
  LogisticFunction() {}
  LogisticFunction(int numAttributes);
  LogisticFunction(istream &);
  double operator()(const Case &);
  double &getA();
  double &getB(int index);
  inline double lambda(const Case &);
  void printOn(ostream &);
  void readFrom(istream &);
private:
  double a;
  BOOM::Array1D<double> b;
};

ostream &operator<<(ostream &,LogisticFunction &);
istream &operator>>(istream &,LogisticFunction &);


double LogisticFunction::lambda(const Case &c)
{
  int n=b.size();
  double sum=a;
  for(int i=0 ; i<n ; ++i)
    sum+=b[i]*c.getContinuous(i);
  return exp(-sum);
}


#endif

