/****************************************************************
 apply-logistic.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include <fstream>
#include "BOOM/String.H"
#include "BOOM/CommandLine.H"
#include "Classifier.H"
#include "../common/Schema.H"
#include "../common/TrainingSet.H"
#include "../common/NamesFile.H"
using namespace std;
using namespace BOOM;


class Application
{
public:
  Application();
  int main(int argc,char *argv[]);

private:
  Schema *schema;
  NamesFile namesFile;
  int numCategories, numVariables, numCases;
  Classifier *classifier;
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
{
  // Process command line
  CommandLine cmd(argc,argv,"");
  if(cmd.numArgs()!=4)
    throw String(
"\n\
apply-logistic <*.names> <*.model> <*.data> <outfile>\n");
  String namesFilename=cmd.arg(0);
  String modelFile=cmd.arg(1);
  String dataFile=cmd.arg(2);
  String outfile=cmd.arg(3);
  
  // Load configuration
  schema=namesFile.processFile(namesFilename);
  numCategories=schema->getNumCategories();
  numVariables=schema->numAttributes();
  
  // Load test data
  TrainingSet *testSet=new TrainingSet(*schema,dataFile,false);
  numCases=testSet->numCases();
  
  // Load model
  ifstream is(modelFile.c_str());
  classifier=Classifier::load(is);
  is.close();
  
  // Generate output
  ofstream os(outfile.c_str());
  for(int i=0 ; i<numCases ; ++i)
    {
      TrainingCase &c=testSet->getCase(i);
      int category=classifier->classify(c);
      os<<category<<endl;
    }
  os.close();
 
  return 0;
}



