/****************************************************************
 Classifier.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include <iostream>
#include "Classifier.H"
#include "BOOM/Array1D.H"
using namespace std;
using namespace BOOM;


/****************************************************************
                        Classifier methods
*****************************************************************/

Classifier *Classifier::load(istream &is)
{
  int t;
  is>>t;
  switch(ClassifierType(t))
    {
    case CL_BINARY:     return new BinaryClassifier(is);
    case CL_MULTICLASS: return new MulticlassClassifier(is);
    }
}



/****************************************************************
                      BinaryClassifier methods
*****************************************************************/
BinaryClassifier::BinaryClassifier(const LogisticFunction &f)
  : f(f)
{
}



BinaryClassifier::BinaryClassifier(istream &is)
  : f(is)
{
}



int BinaryClassifier::classify(const Case &c)
{
  double P=f(c);
  if(P>=0.5) return 0;
  return 1;
}



void BinaryClassifier::save(ostream &os)
{
  os<<CL_BINARY<<endl;
  os<<f<<endl;
}



/****************************************************************
                    MulticlassClassifier methods
*****************************************************************/
MulticlassClassifier::MulticlassClassifier()
{
}



MulticlassClassifier::MulticlassClassifier(istream &is)
{
  f.readFrom(is);
}



void MulticlassClassifier::addFunction(const LogisticFunction &g)
{
  f.push_back(g);
}



int MulticlassClassifier::classify(const Case &c)
{
  int n=f.size(), bestI=0;
  double bestScore=f[0](c);
  for(int i=1 ; i<n ; ++i)
    {
      double P=f[i](c);
      if(P>bestScore)
	{
	  bestI=i;
	  bestScore=P;
	}
    }
  return bestI;
}



void MulticlassClassifier::save(ostream &os)
{
  os<<CL_MULTICLASS<<endl;
  f.printOn(os);
  os<<endl;
}



