/****************************************************************
 Backprop.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Backprop_H
#define INCL_Backprop_H

#include <iostream>
#include "BOOM/Vector.H"
#include "BOOM/Array1D.H"
#include "NeuralCase.H"
#include "NeuralNetwork.H"
#include "TransferFunction.H"
using namespace std;

class Backpropagator
{
  float noiseFactor;
  IntVector *indices;
  int numNeurons;
  ostream *log;
  TransferFunction *f;
  Vector<NeuralCase*> *cases;
  float learningRate;
  FloatArray1D *synapticWeights;
  float bestError;

  void addNoise(float &value);
  void updateNoiseFactor(int iteration,int maxIter);
  void processTrainingCase(NeuralCase &,NeuralNetwork &,
    float learningRate,TransferFunction &,
    IntVector &indices,int numNeurons,float &err);
  void applyCase(NeuralCase &,NeuralNetwork &);
  void processNeuron(int index,NeuralNetwork &nn,
    FloatArray1D &delta,TransferFunction &f,
    float learningRate);
  void computeDelta(int index,NeuralNetwork &,
    FloatArray1D &delta,TransferFunction &);
  void writeToLog(ostream &,int j,float averageError);
public:
  Backpropagator();
  void train(NeuralNetwork &,Vector<NeuralCase*> &,
    float learningRate,TransferFunction &,int maxIterations,
    ostream &,float noiseFactor=0.0);
  void prepareToTrain(NeuralNetwork &nn,float learningRate,
		      ostream &log,TransferFunction &f,
		      Vector<NeuralCase*> &cases);
  float iterate(NeuralNetwork &nn,int j);
  void trainLoop(NeuralNetwork &nn,int maxIterations);
  void recallBestWeights(NeuralNetwork &);
};

#endif
