/****************************************************************
 NetworkBuilder.C
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#include "NetworkBuilder.H"
#include "StandardNeuron.H"
#include "BOOM/Random.H"

NeuralNetwork *NetworkBuilder::buildNetwork(int numInputs,
   int numOutputs,int numHiddenLayers,int neuronsPerLayer)
{
  /* This function builds a network with the given 
     number of input nodes and output nodes, and a 
     series of hidden layers, each having the same 
     number of nodes.  It fills the inputIndices and
     outputIndices arrays with the indices of the 
     input and output neurons, respectively.
   */

  // Create the network object
  NeuralNetwork *nn=new NeuralNetwork;

  // Create the input nodes
  int i;
  for(i=0 ; i<numInputs ; ++i)
    nn->addInputNeuron(new StandardNeuron());
  int prevLayerFrom=0, prevLayerTo=numInputs-1;
    
  // Create the hidden layers
  int layer;
  for(layer=0 ; layer<numHiddenLayers ; ++layer)
    {
      // Create the next hidden layer
      for(i=0 ; i<neuronsPerLayer ; ++i)
	nn->addNeuron(new StandardNeuron());
      int curLayerFrom=prevLayerTo+1;
      int curLayerTo=curLayerFrom+neuronsPerLayer-1;

      // Connect it to the previous layer
      connectLayers(prevLayerFrom,prevLayerTo,
		    curLayerFrom,curLayerTo,*nn);

      // Get ready for the next layer
      prevLayerFrom=curLayerFrom;
      prevLayerTo=curLayerTo;
    }
  
  // Create the output nodes
  for(i=0 ; i<numOutputs ; ++i)
    nn->addOutputNeuron(new StandardNeuron());
  int curLayerFrom=prevLayerTo+1;
  int curLayerTo=curLayerFrom+numOutputs-1;
  connectLayers(prevLayerFrom,prevLayerTo,
		curLayerFrom,curLayerTo,*nn);
  prevLayerFrom=curLayerFrom;
  prevLayerTo=curLayerTo;

  // create a bias node
  nn->addNeuron(new StandardNeuron(1.0));
  int biasNode=curLayerTo+1;
  connectLayers(biasNode,biasNode,
		numInputs,biasNode-1,*nn);

  return nn;
}



void NetworkBuilder::connectLayers(int prevLayerFrom,
   int prevLayerTo,int curLayerFrom,int curLayerTo,
				   NeuralNetwork &nn)
{
  /* Connect all neurons in the previous layer to 
     the neurons in  the current layer.
  */

  int prev, cur;
  for(prev=prevLayerFrom ; prev<=prevLayerTo ; ++prev)
    for(cur=curLayerFrom ; cur<=curLayerTo ; ++cur)
      nn.connect(prev,cur,
		 /*Random0to1()*2-1*/
		 RandomFloat(0.1)-0.05);//see "Machine Learning" p98
}
