/****************************************************************
 NeuralNetwork.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_NeuralNetwork_H
#define INCL_NeuralNetwork_H

#include "BOOM/Stacktrace.H"
#include "BOOM/Vector.H"
#include "BOOM/Array1D.H"
#include <iostream>
using namespace std;
using namespace BOOM;

class Neuron;

class NeuralNetwork
{
  Vector<Neuron*> neurons;
  Vector<int> inputs, outputs;
  bool validIndex(int);
  Neuron *getNeuron(int);//no index checking
  void insertInts(Vector<int> &,int value,int num);
public:
  NeuralNetwork();
  virtual ~NeuralNetwork();
  void addNeuron(Neuron *);
  void addInputNeuron(Neuron *);
  void addOutputNeuron(Neuron *);
  void connect(int from,int to,float weight);
  Neuron &operator[](int);// throws exception
  void setInput(int whichNode,float value);
  void propagate();
  float getOutput(int whichNode);
  int getNumNeurons() const;
  Vector<int> &getInputIndices();
  Vector<int> &getOutputIndices();
  int getNumInputs();
  int getNumOutputs();
  Vector<int> *topologicalSort();//only for feedforward nets!
  void dump(ostream &);
  void saveSelf(ostream &);
  void loadSelf(istream &);
  int countSynapses();
  void copySynapsesOut(FloatArray1D &);
  void copySynapsesIn(FloatArray1D &);
};

#endif
