/****************************************************************
 Neuron.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_NEURON_H
#define INCL_NEURON_H

#include <iostream>
#include "BOOM/Stacktrace.H"
#include "BOOM/Vector.H"
#include "Synapse.H"
#include "NeuralNetwork.H"
#include "NeuronType.H"

// *************************************************
//  		    class Neuron
// *************************************************
class Neuron
{
  float activationValue;
  float combinedInputs;
  Vector<Synapse*> inputs, outputs;
  NeuronType &type;
protected:
  void changeActivationValue(float);
public:
  Neuron(float initialValue,NeuronType &);
  virtual ~Neuron();
  void addInput(const Synapse *);
  void addOutput(const Synapse *);
  int getInDegree() const;
  int getOutDegree() const;
  Synapse &getInput(int);
  Synapse &getOutput(int);
  float getActivationValue() const;
  void update(NeuralNetwork &);
  void setActivationValue(float);
  float getCombinedInputs();
  virtual void saveSelf(ostream &);
  virtual void loadSelf(istream &,NeuralNetwork &);
};


#endif
