/****************************************************************
 Individual.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_Individual_H
#define INCL_Individual_H

#include "BOOM/Vector.H"
#include "TreeNode.H"
#include "CrossoverPoint.H"
#include <iostream>
using namespace std;
using namespace BOOM;


class Individual
{
  Vector<TreeNode*> trees;
  int callCount; // for preventing infinite recursion
  float normalizedFitness; // sum to 1
  float standardizedFitness; // smaller is better
  float adjustedFitness; // 1/(1+standardizedFitness)
  int height;

  TreeNode *loadTree(istream &);
  TreeNode *createNode(NodeType,int arity,istream &);
public:
  Individual(); // calls resetCallCount()
  ~Individual(); // deletes trees
  void decrementCallCount();
  bool canCall();
  void resetCallCount();
  TreeNode *&getADF(int adfNum);
  TreeNode *&getEntryPoint(int entryPointNum=0);
  int getNumTrees() const;
  TreeNode *getTree(int);
  Individual *clone() const;
  void printOn(ostream &) const;
  void save(ostream &) const;
  void load(istream &);
  int getTreeHeight();
  int getCachedHeight();

  inline float getNormalizedFitness() { return normalizedFitness; }
  inline float getStandardizedFitness() { return standardizedFitness; }
  inline float getAdjustedFitness() { return adjustedFitness; }
  inline void setNormalizedFitness(float f) {normalizedFitness=f;}
  inline void setStandardizedFitness(float f) {standardizedFitness=f;}
  inline void setAdjustedFitness(float f) {adjustedFitness=f;}
};

ostream &operator<<(ostream &,const Individual &);

#endif
