/****************************************************************
 TreeNode.H
 william.majoros@duke.edu

 Copyright 2005 William H. Majoros.  This is open-source software,
 governed by the ARTISTIC LICENSE (see www.opensource.org).
 ****************************************************************/
#ifndef INCL_TreeNode_H
#define INCL_TreeNode_H

#include "BOOM/Vector.H"
#include "ActivationRecord.H"
#include <iostream>
using namespace std;
using namespace BOOM;


class Individual;

enum NodeType
{
  TN_MIN,
  TN_MAX,
  TN_FMOD,
  TN_ABS,
  TN_CEIL,
  TN_HYPOT,
  TN_TANH,
  TN_SINH,
  TN_COSH,
  TN_ATANH,
  TN_ASINH,
  TN_ACOSH,
  TN_LOG2,
  TN_LOG10,
  TN_CBRT,
  TN_SQRT,
  TN_ATAN,
  TN_ASIN,
  TN_ACOS,
  TN_TAN,
  TN_POW,
  TN_MOD,
  TN_AND,
  TN_OR,
  TN_NOTEQUALS,
  TN_GREATEREQUALS,
  TN_LESSEQUALS,
  TN_NOT,
  TN_LESS,
  TN_GREATER,
  TN_PLUS,
  TN_MINUS,
  TN_TIMES,
  TN_DIVIDE,
  TN_SIN,
  TN_COS,
  TN_EXP,
  TN_CONSTANT,
  TN_VARIABLE,
  TN_IF,
  TN_CALL,
  TN_LOG,
  TN_EQUALS,
  TN_GREATER_ZERO,
  TN_GREATER_ONE,
  TN_EQUALS_ZERO
};

// ------------------------------------------------------------
class TreeNode
{
private:
  Vector<TreeNode*> children;
  int arity;
protected:
  float leftVal(ActivationRecord &,Individual &);
  float rightVal(ActivationRecord &,Individual &);
  void changeArity(int);
  void save(NodeType,ostream &) const;
public:
  TreeNode(int numChildren=0);
  virtual ~TreeNode(); // deletes its children!
  virtual float eval(ActivationRecord &,Individual &)=0;
  int getNumChildren() const;
  int getArity() const;
  TreeNode *getIthChild(int i) const;
  void setIthChild(int i,TreeNode *);
  virtual TreeNode *instantiate() const=0;
  void addChild(TreeNode *);
  virtual TreeNode *clone() const;
  TreeNode *deepCopy();
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const=0;
  int getTreeHeight() const;
  int countNodes() const;
};

ostream &operator<<(ostream &,const TreeNode &);

// ------------------------------------------------------------

// symbol: min
class MinNode : public TreeNode
{
public:
  MinNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: max
class MaxNode : public TreeNode
{
public:
  MaxNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: fmod
class FmodNode : public TreeNode
{
public:
  FmodNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: abs
class AbsNode : public TreeNode
{
public:
  AbsNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: ceil
class CeilNode : public TreeNode
{
public:
  CeilNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: hypot
class HypotNode : public TreeNode
{
public:
  HypotNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: tanh
class TanhNode : public TreeNode
{
public:
  TanhNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: sinh
class SinhNode : public TreeNode
{
public:
  SinhNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: cosh
class CoshNode : public TreeNode
{
public:
  CoshNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: atanh
class AtanhNode : public TreeNode
{
public:
  AtanhNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: asinh
class AsinhNode : public TreeNode
{
public:
  AsinhNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: acosh
class AcoshNode : public TreeNode
{
public:
  AcoshNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: log2
class Log2Node : public TreeNode
{
public:
  Log2Node();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: log10
class Log10Node : public TreeNode
{
public:
  Log10Node();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: cbrt
class CbrtNode : public TreeNode
{
public:
  CbrtNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: sqrt
class SqrtNode : public TreeNode
{
public:
  SqrtNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: atan
class AtanNode : public TreeNode
{
public:
  AtanNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: asin
class AsinNode : public TreeNode
{
public:
  AsinNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: acos
class AcosNode : public TreeNode
{
public:
  AcosNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: tan
class TanNode : public TreeNode
{
public:
  TanNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: ^
class PowNode : public TreeNode
{
public:
  PowNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: %
class ModNode : public TreeNode
{
public:
  ModNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: &&
class AndNode : public TreeNode
{
public:
  AndNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: ||
class OrNode : public TreeNode
{
public:
  OrNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: !=
class NotEqualsNode : public TreeNode
{
public:
  NotEqualsNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: >=
class GreaterEqualsNode : public TreeNode
{
public:
  GreaterEqualsNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: <=
class LessEqualsNode : public TreeNode
{
public:
  LessEqualsNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: !
class NotNode : public TreeNode
{
public:
  NotNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: <
class LessNode : public TreeNode
{
public:
  LessNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: >
class GreaterNode : public TreeNode
{
public:
  GreaterNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: +
class PlusNode : public TreeNode
{
public:
  PlusNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: -
class MinusNode : public TreeNode
{
public:
  MinusNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: *
class TimesNode : public TreeNode
{
public:
  TimesNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: /
class DivideNode : public TreeNode
{
public:
  DivideNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: sin
class SinNode : public TreeNode
{
public:
  SinNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: cos
class CosNode : public TreeNode
{
public:
  CosNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: exp
class ExpNode : public TreeNode
{
public:
  ExpNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

class ConstantNode : public TreeNode
{
  static float minVal, maxVal;
  float value;

  ConstantNode();
public:
  ConstantNode(float minVal,float maxVal);
  ConstantNode(float);
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual TreeNode *clone() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

class VariableNode : public TreeNode
{
  int whichParm;

public:
  VariableNode();
  VariableNode(int whichParm);
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual TreeNode *clone() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: if
class IfNode : public TreeNode // tests if expr is pos
{
public:
  IfNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: call
class CallNode : public TreeNode
{
  int whichADF;

public:
  CallNode();
  CallNode(int whichADF,int arity);
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual TreeNode *clone() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: log
class LogNode : public TreeNode
{
public:
  LogNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: =
class EqualsNode : public TreeNode
{
public:
  EqualsNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: >0
class GreaterThanZeroNode : public TreeNode
{
public:
  GreaterThanZeroNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: >1
class GreaterThanOneNode : public TreeNode
{
public:
  GreaterThanOneNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};

// ------------------------------------------------------------

// symbol: =0
class EqualsZeroNode : public TreeNode
{
public:
  EqualsZeroNode();
  virtual float eval(ActivationRecord &,Individual &);
  virtual TreeNode *instantiate() const;
  virtual void printOn(ostream &) const;
  virtual void save(ostream &) const;
};



#endif
